"""The dvm commands module.

These commands are used in the following models:
DPO4K, DPO4KB, MDO3, MDO3K, MDO4K, MDO4KB, MDO4KC, MSO4K, MSO4KB

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - DVM {RESET}
    - DVM:AUTORange {0|1|OFF|ON}
    - DVM:AUTORange?
    - DVM:DISPLAYSTYle {FULl|MINimum}
    - DVM:DISPLAYSTYle?
    - DVM:MEASUrement:FREQuency?
    - DVM:MEASUrement:HIStory:AVErage?
    - DVM:MEASUrement:HIStory:MAXimum?
    - DVM:MEASUrement:HIStory:MINImum?
    - DVM:MEASUrement:INFMAXimum?
    - DVM:MEASUrement:INFMINimum?
    - DVM:MEASUrement:VALue?
    - DVM:MODe {ACRMS|ACDCRMS|DC|FREQuency|OFF}
    - DVM:MODe?
    - DVM:SOUrce {CH<x>}
    - DVM:SOUrce?
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class DvmSource(SCPICmdWrite, SCPICmdRead):
    """The ``DVM:SOUrce`` command.

    **Description:**
        - This command sets (or queries) the source for the DVM.

    **Usage:**
        - Using the ``.query()`` method will send the ``DVM:SOUrce?`` query.
        - Using the ``.verify(value)`` method will send the ``DVM:SOUrce?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DVM:SOUrce value`` command.

    **SCPI Syntax:**

    ::

        - DVM:SOUrce {CH<x>}
        - DVM:SOUrce?

    **Info:**
        - ``CH<x>`` specify which channel to use as the source for the DVM.
    """


class DvmMode(SCPICmdWrite, SCPICmdRead):
    """The ``DVM:MODe`` command.

    **Description:**
        - Specifies (or queries) the mode to use for the Digital Voltmeter (ACRMS, ACDCRMS, DC,
          Frequency, or OFF).

    **Usage:**
        - Using the ``.query()`` method will send the ``DVM:MODe?`` query.
        - Using the ``.verify(value)`` method will send the ``DVM:MODe?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DVM:MODe value`` command.

    **SCPI Syntax:**

    ::

        - DVM:MODe {ACRMS|ACDCRMS|DC|FREQuency|OFF}
        - DVM:MODe?

    **Info:**
        - ``ACRMS`` - displays the root-mean-square value of the acquired data, with the DC
          component removed.
        - ``ACDCRMS`` - displays the RMS value of the acquired data.
        - ``DC`` - displays the DC value of the acquired data.
        - ``FREQuency`` - displays the frequency counter measurement.
        - ``OFF``
    """


class DvmMeasurementValue(SCPICmdRead):
    """The ``DVM:MEASUrement:VALue`` command.

    **Description:**
        - Returns the DVM readout value (the largest displayed value at the top of the DVM screen).

    **Usage:**
        - Using the ``.query()`` method will send the ``DVM:MEASUrement:VALue?`` query.
        - Using the ``.verify(value)`` method will send the ``DVM:MEASUrement:VALue?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - DVM:MEASUrement:VALue?
    """


class DvmMeasurementInfminimum(SCPICmdRead):
    """The ``DVM:MEASUrement:INFMINimum`` command.

    **Description:**
        - Returns the minimum readout value of the DVM over the entire time that the DVM has been on
          since the last change using the ``DVM:MODE`` or ``DVM:SOURCE`` commands or DVM RESET.

    **Usage:**
        - Using the ``.query()`` method will send the ``DVM:MEASUrement:INFMINimum?`` query.
        - Using the ``.verify(value)`` method will send the ``DVM:MEASUrement:INFMINimum?`` query
          and raise an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - DVM:MEASUrement:INFMINimum?
    """


class DvmMeasurementInfmaximum(SCPICmdRead):
    """The ``DVM:MEASUrement:INFMAXimum`` command.

    **Description:**
        - Returns the maximum DVM readout value over the entire time that the DVM has been on since
          the last change using the ``DVM:MODE`` or ``DVM:SOURCE`` commands or DVM RESET.

    **Usage:**
        - Using the ``.query()`` method will send the ``DVM:MEASUrement:INFMAXimum?`` query.
        - Using the ``.verify(value)`` method will send the ``DVM:MEASUrement:INFMAXimum?`` query
          and raise an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - DVM:MEASUrement:INFMAXimum?
    """


class DvmMeasurementHistoryMinimum(SCPICmdRead):
    """The ``DVM:MEASUrement:HIStory:MINImum`` command.

    **Description:**
        - Returns the minimum readout value for the DVM over the history period. The history period
          is a constant period of 5 seconds.

    **Usage:**
        - Using the ``.query()`` method will send the ``DVM:MEASUrement:HIStory:MINImum?`` query.
        - Using the ``.verify(value)`` method will send the ``DVM:MEASUrement:HIStory:MINImum?``
          query and raise an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - DVM:MEASUrement:HIStory:MINImum?
    """


class DvmMeasurementHistoryMaximum(SCPICmdRead):
    """The ``DVM:MEASUrement:HIStory:MAXimum`` command.

    **Description:**
        - Returns the maximum readout value for the DVM function over the history period. The
          history period is a constant period of 5 seconds.

    **Usage:**
        - Using the ``.query()`` method will send the ``DVM:MEASUrement:HIStory:MAXimum?`` query.
        - Using the ``.verify(value)`` method will send the ``DVM:MEASUrement:HIStory:MAXimum?``
          query and raise an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - DVM:MEASUrement:HIStory:MAXimum?
    """


class DvmMeasurementHistoryAverage(SCPICmdRead):
    """The ``DVM:MEASUrement:HIStory:AVErage`` command.

    **Description:**
        - Returns the average DVM readout value over the history period. The history period is a
          constant period of 5 seconds.

    **Usage:**
        - Using the ``.query()`` method will send the ``DVM:MEASUrement:HIStory:AVErage?`` query.
        - Using the ``.verify(value)`` method will send the ``DVM:MEASUrement:HIStory:AVErage?``
          query and raise an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - DVM:MEASUrement:HIStory:AVErage?
    """


class DvmMeasurementHistory(SCPICmdRead):
    """The ``DVM:MEASUrement:HIStory`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``DVM:MEASUrement:HIStory?`` query.
        - Using the ``.verify(value)`` method will send the ``DVM:MEASUrement:HIStory?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.average``: The ``DVM:MEASUrement:HIStory:AVErage`` command.
        - ``.maximum``: The ``DVM:MEASUrement:HIStory:MAXimum`` command.
        - ``.minimum``: The ``DVM:MEASUrement:HIStory:MINImum`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._average = DvmMeasurementHistoryAverage(device, f"{self._cmd_syntax}:AVErage")
        self._maximum = DvmMeasurementHistoryMaximum(device, f"{self._cmd_syntax}:MAXimum")
        self._minimum = DvmMeasurementHistoryMinimum(device, f"{self._cmd_syntax}:MINImum")

    @property
    def average(self) -> DvmMeasurementHistoryAverage:
        """Return the ``DVM:MEASUrement:HIStory:AVErage`` command.

        **Description:**
            - Returns the average DVM readout value over the history period. The history period is a
              constant period of 5 seconds.

        **Usage:**
            - Using the ``.query()`` method will send the ``DVM:MEASUrement:HIStory:AVErage?``
              query.
            - Using the ``.verify(value)`` method will send the ``DVM:MEASUrement:HIStory:AVErage?``
              query and raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - DVM:MEASUrement:HIStory:AVErage?
        """
        return self._average

    @property
    def maximum(self) -> DvmMeasurementHistoryMaximum:
        """Return the ``DVM:MEASUrement:HIStory:MAXimum`` command.

        **Description:**
            - Returns the maximum readout value for the DVM function over the history period. The
              history period is a constant period of 5 seconds.

        **Usage:**
            - Using the ``.query()`` method will send the ``DVM:MEASUrement:HIStory:MAXimum?``
              query.
            - Using the ``.verify(value)`` method will send the ``DVM:MEASUrement:HIStory:MAXimum?``
              query and raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - DVM:MEASUrement:HIStory:MAXimum?
        """
        return self._maximum

    @property
    def minimum(self) -> DvmMeasurementHistoryMinimum:
        """Return the ``DVM:MEASUrement:HIStory:MINImum`` command.

        **Description:**
            - Returns the minimum readout value for the DVM over the history period. The history
              period is a constant period of 5 seconds.

        **Usage:**
            - Using the ``.query()`` method will send the ``DVM:MEASUrement:HIStory:MINImum?``
              query.
            - Using the ``.verify(value)`` method will send the ``DVM:MEASUrement:HIStory:MINImum?``
              query and raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - DVM:MEASUrement:HIStory:MINImum?
        """
        return self._minimum


class DvmMeasurementFrequency(SCPICmdRead):
    """The ``DVM:MEASUrement:FREQuency`` command.

    **Description:**
        - This command returns the current frequency value for the DVM.

    **Usage:**
        - Using the ``.query()`` method will send the ``DVM:MEASUrement:FREQuency?`` query.
        - Using the ``.verify(value)`` method will send the ``DVM:MEASUrement:FREQuency?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - DVM:MEASUrement:FREQuency?
    """


class DvmMeasurement(SCPICmdRead):
    """The ``DVM:MEASUrement`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``DVM:MEASUrement?`` query.
        - Using the ``.verify(value)`` method will send the ``DVM:MEASUrement?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.frequency``: The ``DVM:MEASUrement:FREQuency`` command.
        - ``.history``: The ``DVM:MEASUrement:HIStory`` command tree.
        - ``.infmaximum``: The ``DVM:MEASUrement:INFMAXimum`` command.
        - ``.infminimum``: The ``DVM:MEASUrement:INFMINimum`` command.
        - ``.value``: The ``DVM:MEASUrement:VALue`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._frequency = DvmMeasurementFrequency(device, f"{self._cmd_syntax}:FREQuency")
        self._history = DvmMeasurementHistory(device, f"{self._cmd_syntax}:HIStory")
        self._infmaximum = DvmMeasurementInfmaximum(device, f"{self._cmd_syntax}:INFMAXimum")
        self._infminimum = DvmMeasurementInfminimum(device, f"{self._cmd_syntax}:INFMINimum")
        self._value = DvmMeasurementValue(device, f"{self._cmd_syntax}:VALue")

    @property
    def frequency(self) -> DvmMeasurementFrequency:
        """Return the ``DVM:MEASUrement:FREQuency`` command.

        **Description:**
            - This command returns the current frequency value for the DVM.

        **Usage:**
            - Using the ``.query()`` method will send the ``DVM:MEASUrement:FREQuency?`` query.
            - Using the ``.verify(value)`` method will send the ``DVM:MEASUrement:FREQuency?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - DVM:MEASUrement:FREQuency?
        """
        return self._frequency

    @property
    def history(self) -> DvmMeasurementHistory:
        """Return the ``DVM:MEASUrement:HIStory`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``DVM:MEASUrement:HIStory?`` query.
            - Using the ``.verify(value)`` method will send the ``DVM:MEASUrement:HIStory?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.average``: The ``DVM:MEASUrement:HIStory:AVErage`` command.
            - ``.maximum``: The ``DVM:MEASUrement:HIStory:MAXimum`` command.
            - ``.minimum``: The ``DVM:MEASUrement:HIStory:MINImum`` command.
        """
        return self._history

    @property
    def infmaximum(self) -> DvmMeasurementInfmaximum:
        """Return the ``DVM:MEASUrement:INFMAXimum`` command.

        **Description:**
            - Returns the maximum DVM readout value over the entire time that the DVM has been on
              since the last change using the ``DVM:MODE`` or ``DVM:SOURCE`` commands or DVM RESET.

        **Usage:**
            - Using the ``.query()`` method will send the ``DVM:MEASUrement:INFMAXimum?`` query.
            - Using the ``.verify(value)`` method will send the ``DVM:MEASUrement:INFMAXimum?``
              query and raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - DVM:MEASUrement:INFMAXimum?
        """
        return self._infmaximum

    @property
    def infminimum(self) -> DvmMeasurementInfminimum:
        """Return the ``DVM:MEASUrement:INFMINimum`` command.

        **Description:**
            - Returns the minimum readout value of the DVM over the entire time that the DVM has
              been on since the last change using the ``DVM:MODE`` or ``DVM:SOURCE`` commands or DVM
              RESET.

        **Usage:**
            - Using the ``.query()`` method will send the ``DVM:MEASUrement:INFMINimum?`` query.
            - Using the ``.verify(value)`` method will send the ``DVM:MEASUrement:INFMINimum?``
              query and raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - DVM:MEASUrement:INFMINimum?
        """
        return self._infminimum

    @property
    def value(self) -> DvmMeasurementValue:
        """Return the ``DVM:MEASUrement:VALue`` command.

        **Description:**
            - Returns the DVM readout value (the largest displayed value at the top of the DVM
              screen).

        **Usage:**
            - Using the ``.query()`` method will send the ``DVM:MEASUrement:VALue?`` query.
            - Using the ``.verify(value)`` method will send the ``DVM:MEASUrement:VALue?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - DVM:MEASUrement:VALue?
        """
        return self._value


class DvmDisplaystyle(SCPICmdWrite, SCPICmdRead):
    """The ``DVM:DISPLAYSTYle`` command.

    **Description:**
        - Sets (or queries) the display style for the Digital Voltmeter.

    **Usage:**
        - Using the ``.query()`` method will send the ``DVM:DISPLAYSTYle?`` query.
        - Using the ``.verify(value)`` method will send the ``DVM:DISPLAYSTYle?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DVM:DISPLAYSTYle value`` command.

    **SCPI Syntax:**

    ::

        - DVM:DISPLAYSTYle {FULl|MINimum}
        - DVM:DISPLAYSTYle?

    **Info:**
        - ``FULl`` sets the display style for the DVM to Full.
        - ``MINimum`` sets the display style for the DVM to Minimum.
    """


class DvmAutorange(SCPICmdWrite, SCPICmdRead):
    """The ``DVM:AUTORange`` command.

    **Description:**
        - Sets (or queries) the autorange state for the Digital Voltmeter.

    **Usage:**
        - Using the ``.query()`` method will send the ``DVM:AUTORange?`` query.
        - Using the ``.verify(value)`` method will send the ``DVM:AUTORange?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DVM:AUTORange value`` command.

    **SCPI Syntax:**

    ::

        - DVM:AUTORange {0|1|OFF|ON}
        - DVM:AUTORange?

    **Info:**
        - ``1`` or ON turns on autorange for the Digital Voltmeter.
        - ``0`` or OFF turns autorange off.
    """


class Dvm(SCPICmdWrite, SCPICmdRead):
    """The ``DVM`` command.

    **Description:**
        - Resets the Digital Voltmeter measurements and history.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``DVM value`` command.

    **SCPI Syntax:**

    ::

        - DVM {RESET}

    Properties:
        - ``.autorange``: The ``DVM:AUTORange`` command.
        - ``.displaystyle``: The ``DVM:DISPLAYSTYle`` command.
        - ``.measurement``: The ``DVM:MEASUrement`` command tree.
        - ``.mode``: The ``DVM:MODe`` command.
        - ``.source``: The ``DVM:SOUrce`` command.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "DVM") -> None:
        super().__init__(device, cmd_syntax)
        self._autorange = DvmAutorange(device, f"{self._cmd_syntax}:AUTORange")
        self._displaystyle = DvmDisplaystyle(device, f"{self._cmd_syntax}:DISPLAYSTYle")
        self._measurement = DvmMeasurement(device, f"{self._cmd_syntax}:MEASUrement")
        self._mode = DvmMode(device, f"{self._cmd_syntax}:MODe")
        self._source = DvmSource(device, f"{self._cmd_syntax}:SOUrce")

    @property
    def autorange(self) -> DvmAutorange:
        """Return the ``DVM:AUTORange`` command.

        **Description:**
            - Sets (or queries) the autorange state for the Digital Voltmeter.

        **Usage:**
            - Using the ``.query()`` method will send the ``DVM:AUTORange?`` query.
            - Using the ``.verify(value)`` method will send the ``DVM:AUTORange?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``DVM:AUTORange value`` command.

        **SCPI Syntax:**

        ::

            - DVM:AUTORange {0|1|OFF|ON}
            - DVM:AUTORange?

        **Info:**
            - ``1`` or ON turns on autorange for the Digital Voltmeter.
            - ``0`` or OFF turns autorange off.
        """
        return self._autorange

    @property
    def displaystyle(self) -> DvmDisplaystyle:
        """Return the ``DVM:DISPLAYSTYle`` command.

        **Description:**
            - Sets (or queries) the display style for the Digital Voltmeter.

        **Usage:**
            - Using the ``.query()`` method will send the ``DVM:DISPLAYSTYle?`` query.
            - Using the ``.verify(value)`` method will send the ``DVM:DISPLAYSTYle?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``DVM:DISPLAYSTYle value`` command.

        **SCPI Syntax:**

        ::

            - DVM:DISPLAYSTYle {FULl|MINimum}
            - DVM:DISPLAYSTYle?

        **Info:**
            - ``FULl`` sets the display style for the DVM to Full.
            - ``MINimum`` sets the display style for the DVM to Minimum.
        """
        return self._displaystyle

    @property
    def measurement(self) -> DvmMeasurement:
        """Return the ``DVM:MEASUrement`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``DVM:MEASUrement?`` query.
            - Using the ``.verify(value)`` method will send the ``DVM:MEASUrement?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.frequency``: The ``DVM:MEASUrement:FREQuency`` command.
            - ``.history``: The ``DVM:MEASUrement:HIStory`` command tree.
            - ``.infmaximum``: The ``DVM:MEASUrement:INFMAXimum`` command.
            - ``.infminimum``: The ``DVM:MEASUrement:INFMINimum`` command.
            - ``.value``: The ``DVM:MEASUrement:VALue`` command.
        """
        return self._measurement

    @property
    def mode(self) -> DvmMode:
        """Return the ``DVM:MODe`` command.

        **Description:**
            - Specifies (or queries) the mode to use for the Digital Voltmeter (ACRMS, ACDCRMS, DC,
              Frequency, or OFF).

        **Usage:**
            - Using the ``.query()`` method will send the ``DVM:MODe?`` query.
            - Using the ``.verify(value)`` method will send the ``DVM:MODe?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``DVM:MODe value`` command.

        **SCPI Syntax:**

        ::

            - DVM:MODe {ACRMS|ACDCRMS|DC|FREQuency|OFF}
            - DVM:MODe?

        **Info:**
            - ``ACRMS`` - displays the root-mean-square value of the acquired data, with the DC
              component removed.
            - ``ACDCRMS`` - displays the RMS value of the acquired data.
            - ``DC`` - displays the DC value of the acquired data.
            - ``FREQuency`` - displays the frequency counter measurement.
            - ``OFF``
        """
        return self._mode

    @property
    def source(self) -> DvmSource:
        """Return the ``DVM:SOUrce`` command.

        **Description:**
            - This command sets (or queries) the source for the DVM.

        **Usage:**
            - Using the ``.query()`` method will send the ``DVM:SOUrce?`` query.
            - Using the ``.verify(value)`` method will send the ``DVM:SOUrce?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``DVM:SOUrce value`` command.

        **SCPI Syntax:**

        ::

            - DVM:SOUrce {CH<x>}
            - DVM:SOUrce?

        **Info:**
            - ``CH<x>`` specify which channel to use as the source for the DVM.
        """
        return self._source



# pylint: disable=line-too-long
"""The pictbridge commands module.

These commands are used in the following models:
DPO2K, DPO2KB, MSO2K, MSO2KB

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - PICTBridge:DATEPrint {DEFLT|OFF|ON}
    - PICTBridge:DEFault
    - PICTBridge:IDPrint {DEFLT|OFF|ON}
    - PICTBridge:IDPrint?
    - PICTBridge:IMAGESize {DEFLT|IN2P5BY3P25|L|IN4BY6|L2|IN8BY10|L4|E|CARD|HAGAKIPC|CM6BY8|CM7BY10|CM9BY13|CM10BY15|CM13BY18|CM15BY21|CM18BY24| A4|LETTER}
    - PICTBridge:PAPERSize {DEFLT|L|L2|HAGAKIPCARD|MM54BY86|MM100BY150|IN4BY6|IN8BY10|LETTER|IN11BY17|A0|A1|A2|A3|A4|A5|A6|A7|A8|A9|B0|B1|B2|B3| B4|B5|B6|B7|B8|B9|ROLL89MM|ROLL127MM|ROLL100MM|ROLL210MM}
    - PICTBridge:PAPERType {DEFLT|PLAIN|PHOTO|FASTPHOTO}
    - PICTBridge:PRINTQual {DEFLT|NRMAL|FINE|DRAFT}
"""  # noqa: E501
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdRead, SCPICmdWrite, SCPICmdWriteNoArguments

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class PictbridgePrintqual(SCPICmdWrite):
    """The ``PICTBridge:PRINTQual`` command.

    **Description:**
        - Sets or returns the output print quality.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``PICTBridge:PRINTQual value`` command.

    **SCPI Syntax:**

    ::

        - PICTBridge:PRINTQual {DEFLT|NRMAL|FINE|DRAFT}

    **Info:**
        - ``DEFLT`` for the default quality print.
        - ``NRMAL`` for a normal quality print.
        - ``FINE`` for a fine quality print.
        - ``DRAFT`` for a draft quality print.
    """


class PictbridgePapertype(SCPICmdWrite):
    """The ``PICTBridge:PAPERType`` command.

    **Description:**
        - Sets or returns the paper type.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``PICTBridge:PAPERType value`` command.

    **SCPI Syntax:**

    ::

        - PICTBridge:PAPERType {DEFLT|PLAIN|PHOTO|FASTPHOTO}

    **Info:**
        - ``DEFLT`` for a default print paper type.
        - ``PLAIN`` for a plain print paper type.
        - ``PHOTO`` for a photo print paper type.
        - ``FASTPHOTO`` for a fastphoto print paper type.
    """


class PictbridgePapersize(SCPICmdWrite):
    """The ``PICTBridge:PAPERSize`` command.

    **Description:**
        - Sets the output print paper size.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``PICTBridge:PAPERSize value`` command.

    **SCPI Syntax:**

    ::

        - PICTBridge:PAPERSize {DEFLT|L|L2|HAGAKIPCARD|MM54BY86|MM100BY150|IN4BY6|IN8BY10|LETTER|IN11BY17|A0|A1|A2|A3|A4|A5|A6|A7|A8|A9|B0|B1|B2|B3| B4|B5|B6|B7|B8|B9|ROLL89MM|ROLL127MM|ROLL100MM|ROLL210MM}

    **Info:**
        - ``DEFLT`` for a default paper size.
        - ``L`` for a paper size L.
        - ``L2`` for a paper size 2L.
        - ``HAGAKIPCARD`` for a paper size Hagaki.
        - ``MM54BY86`` for a card paper size.
        - ``MM100BY150`` for paper size of 100``*150 MM``.
        - ``IN4BY6`` for a paper size of 4``*6``.
        - ``IN8BY10`` for a paper size of 8``*10``.
        - ``LETTER`` for a letter paper size.
        - ``IN11BY17`` for a paper size of 11``*17``.
        - ``A0`` for a A0 paper size.
        - ``A1`` for a A1 paper size.
        - ``A2`` for a A2 paper size.
        - ``A3`` for a A3 paper size.
        - ``A4`` for a A4 paper size.
        - ``A5`` for a A5 paper size.
        - ``A6`` for a A6 paper size.
        - ``A7`` for a A7 paper size.
        - ``A8`` for a A8 paper size.
        - ``A9`` for a A9 paper size.
        - ``B0`` for a B0 paper size.
        - ``B1`` for a B1 paper size.
        - ``B2`` for a B2 paper size.
        - ``B3`` for a B3 paper size.
        - ``B4`` for a B4 paper size.
        - ``B5`` for a B5 paper size.
        - ``B6`` for a B6 paper size.
        - ``B7`` for a B7 paper size.
        - ``B8`` for a B8 paper size.
        - ``B9`` for a B9 paper size.
        - ``ROLL89MM`` for a 89 MM Roll paper size.
        - ``ROLL127MM`` for a 127 MM Roll paper size.
        - ``ROLL100MM`` for a 100 MM Roll paper size.
        - ``ROLL210MM`` for a 210 MM Roll paper size.
    """  # noqa: E501


class PictbridgeImagesize(SCPICmdWrite):
    """The ``PICTBridge:IMAGESize`` command.

    **Description:**
        - Sets or returns the image print size.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``PICTBridge:IMAGESize value`` command.

    **SCPI Syntax:**

    ::

        - PICTBridge:IMAGESize {DEFLT|IN2P5BY3P25|L|IN4BY6|L2|IN8BY10|L4|E|CARD|HAGAKIPC|CM6BY8|CM7BY10|CM9BY13|CM10BY15|CM13BY18|CM15BY21|CM18BY24| A4|LETTER}

    **Info:**
        - ``DEFLT`` for a default image print size.
        - ``IN2P5BY3P25`` for a ``2_5````*3_25`` image print size.
        - ``L`` for a ``3_5````*5`` image print size.
        - ``IN4BY6`` for a 4``*6`` image print size.
        - ``L2`` for a 5``*7`` image print size.
        - ``IN8BY10`` for a 8``*10`` image print size.
        - ``L4`` for a 254 MM``*178 MM`` image print size.
        - ``E`` for a 110 MM``*74 MM`` image print size.
        - ``CARD`` for a 89 MM``*55MM`` image print size.
        - ``HAGAKIPC`` for a 100 MM``*148 MM`` image print size.
        - ``CM6BY8`` for a 6 CM``*8 CM`` image print size.
        - ``CM7BY10`` for a 7 CM``*10 CM`` image print size.
        - ``CM9BY13`` for a 9 CM``*13 CM`` image print size.
        - ``CM10BY15`` for a 10 CM``*15 CM`` image print size.
        - ``CM13BY18`` or a 13 CM``*18 CM`` image print size.
        - ``CM15BY21`` for a 15 CM``*21 CM`` image print size.
        - ``CM18BY24`` for a 18 CM``*24 CM`` image print size.
        - ``A4`` for a A4 image print size.
        - ``LETTER`` for a Letter image print size.
    """  # noqa: E501


class PictbridgeIdprint(SCPICmdWrite, SCPICmdRead):
    """The ``PICTBridge:IDPrint`` command.

    **Description:**
        - Enables or disables printing the oscilloscope model and serial number on the print output.

    **Usage:**
        - Using the ``.query()`` method will send the ``PICTBridge:IDPrint?`` query.
        - Using the ``.verify(value)`` method will send the ``PICTBridge:IDPrint?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``PICTBridge:IDPrint value`` command.

    **SCPI Syntax:**

    ::

        - PICTBridge:IDPrint {DEFLT|OFF|ON}
        - PICTBridge:IDPrint?

    **Info:**
        - ``DEFLT`` is the default setting of the printer.
        - ``ON`` enables the oscilloscope model and serial number print on the print output.
        - ``OFF`` disables the oscilloscope model and serial number print on the print output.
    """


class PictbridgeDefault(SCPICmdWriteNoArguments):
    """The ``PICTBridge:DEFault`` command.

    **Description:**
        - Sets the arguments for all PictBridge commands to their default values. The default values
          are same as printer default settings.

    **Usage:**
        - Using the ``.write()`` method will send the ``PICTBridge:DEFault`` command.

    **SCPI Syntax:**

    ::

        - PICTBridge:DEFault
    """


class PictbridgeDateprint(SCPICmdWrite):
    """The ``PICTBridge:DATEPrint`` command.

    **Description:**
        - Enables or disables printing the date on the print output.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``PICTBridge:DATEPrint value`` command.

    **SCPI Syntax:**

    ::

        - PICTBridge:DATEPrint {DEFLT|OFF|ON}

    **Info:**
        - ``DEFLT`` is the default setting of the printer.
        - ``ON`` enables the date print on the print output.
        - ``OFF`` disables the date print on the print output.
    """


class Pictbridge(SCPICmdRead):
    """The ``PICTBridge`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``PICTBridge?`` query.
        - Using the ``.verify(value)`` method will send the ``PICTBridge?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.dateprint``: The ``PICTBridge:DATEPrint`` command.
        - ``.default``: The ``PICTBridge:DEFault`` command.
        - ``.idprint``: The ``PICTBridge:IDPrint`` command.
        - ``.imagesize``: The ``PICTBridge:IMAGESize`` command.
        - ``.papersize``: The ``PICTBridge:PAPERSize`` command.
        - ``.papertype``: The ``PICTBridge:PAPERType`` command.
        - ``.printqual``: The ``PICTBridge:PRINTQual`` command.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "PICTBridge") -> None:
        super().__init__(device, cmd_syntax)
        self._dateprint = PictbridgeDateprint(device, f"{self._cmd_syntax}:DATEPrint")
        self._default = PictbridgeDefault(device, f"{self._cmd_syntax}:DEFault")
        self._idprint = PictbridgeIdprint(device, f"{self._cmd_syntax}:IDPrint")
        self._imagesize = PictbridgeImagesize(device, f"{self._cmd_syntax}:IMAGESize")
        self._papersize = PictbridgePapersize(device, f"{self._cmd_syntax}:PAPERSize")
        self._papertype = PictbridgePapertype(device, f"{self._cmd_syntax}:PAPERType")
        self._printqual = PictbridgePrintqual(device, f"{self._cmd_syntax}:PRINTQual")

    @property
    def dateprint(self) -> PictbridgeDateprint:
        """Return the ``PICTBridge:DATEPrint`` command.

        **Description:**
            - Enables or disables printing the date on the print output.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``PICTBridge:DATEPrint value``
              command.

        **SCPI Syntax:**

        ::

            - PICTBridge:DATEPrint {DEFLT|OFF|ON}

        **Info:**
            - ``DEFLT`` is the default setting of the printer.
            - ``ON`` enables the date print on the print output.
            - ``OFF`` disables the date print on the print output.
        """
        return self._dateprint

    @property
    def default(self) -> PictbridgeDefault:
        """Return the ``PICTBridge:DEFault`` command.

        **Description:**
            - Sets the arguments for all PictBridge commands to their default values. The default
              values are same as printer default settings.

        **Usage:**
            - Using the ``.write()`` method will send the ``PICTBridge:DEFault`` command.

        **SCPI Syntax:**

        ::

            - PICTBridge:DEFault
        """
        return self._default

    @property
    def idprint(self) -> PictbridgeIdprint:
        """Return the ``PICTBridge:IDPrint`` command.

        **Description:**
            - Enables or disables printing the oscilloscope model and serial number on the print
              output.

        **Usage:**
            - Using the ``.query()`` method will send the ``PICTBridge:IDPrint?`` query.
            - Using the ``.verify(value)`` method will send the ``PICTBridge:IDPrint?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``PICTBridge:IDPrint value`` command.

        **SCPI Syntax:**

        ::

            - PICTBridge:IDPrint {DEFLT|OFF|ON}
            - PICTBridge:IDPrint?

        **Info:**
            - ``DEFLT`` is the default setting of the printer.
            - ``ON`` enables the oscilloscope model and serial number print on the print output.
            - ``OFF`` disables the oscilloscope model and serial number print on the print output.
        """
        return self._idprint

    @property
    def imagesize(self) -> PictbridgeImagesize:
        """Return the ``PICTBridge:IMAGESize`` command.

        **Description:**
            - Sets or returns the image print size.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``PICTBridge:IMAGESize value``
              command.

        **SCPI Syntax:**

        ::

            - PICTBridge:IMAGESize {DEFLT|IN2P5BY3P25|L|IN4BY6|L2|IN8BY10|L4|E|CARD|HAGAKIPC|CM6BY8|CM7BY10|CM9BY13|CM10BY15|CM13BY18|CM15BY21|CM18BY24| A4|LETTER}

        **Info:**
            - ``DEFLT`` for a default image print size.
            - ``IN2P5BY3P25`` for a ``2_5````*3_25`` image print size.
            - ``L`` for a ``3_5````*5`` image print size.
            - ``IN4BY6`` for a 4``*6`` image print size.
            - ``L2`` for a 5``*7`` image print size.
            - ``IN8BY10`` for a 8``*10`` image print size.
            - ``L4`` for a 254 MM``*178 MM`` image print size.
            - ``E`` for a 110 MM``*74 MM`` image print size.
            - ``CARD`` for a 89 MM``*55MM`` image print size.
            - ``HAGAKIPC`` for a 100 MM``*148 MM`` image print size.
            - ``CM6BY8`` for a 6 CM``*8 CM`` image print size.
            - ``CM7BY10`` for a 7 CM``*10 CM`` image print size.
            - ``CM9BY13`` for a 9 CM``*13 CM`` image print size.
            - ``CM10BY15`` for a 10 CM``*15 CM`` image print size.
            - ``CM13BY18`` or a 13 CM``*18 CM`` image print size.
            - ``CM15BY21`` for a 15 CM``*21 CM`` image print size.
            - ``CM18BY24`` for a 18 CM``*24 CM`` image print size.
            - ``A4`` for a A4 image print size.
            - ``LETTER`` for a Letter image print size.
        """  # noqa: E501
        return self._imagesize

    @property
    def papersize(self) -> PictbridgePapersize:
        """Return the ``PICTBridge:PAPERSize`` command.

        **Description:**
            - Sets the output print paper size.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``PICTBridge:PAPERSize value``
              command.

        **SCPI Syntax:**

        ::

            - PICTBridge:PAPERSize {DEFLT|L|L2|HAGAKIPCARD|MM54BY86|MM100BY150|IN4BY6|IN8BY10|LETTER|IN11BY17|A0|A1|A2|A3|A4|A5|A6|A7|A8|A9|B0|B1|B2|B3| B4|B5|B6|B7|B8|B9|ROLL89MM|ROLL127MM|ROLL100MM|ROLL210MM}

        **Info:**
            - ``DEFLT`` for a default paper size.
            - ``L`` for a paper size L.
            - ``L2`` for a paper size 2L.
            - ``HAGAKIPCARD`` for a paper size Hagaki.
            - ``MM54BY86`` for a card paper size.
            - ``MM100BY150`` for paper size of 100``*150 MM``.
            - ``IN4BY6`` for a paper size of 4``*6``.
            - ``IN8BY10`` for a paper size of 8``*10``.
            - ``LETTER`` for a letter paper size.
            - ``IN11BY17`` for a paper size of 11``*17``.
            - ``A0`` for a A0 paper size.
            - ``A1`` for a A1 paper size.
            - ``A2`` for a A2 paper size.
            - ``A3`` for a A3 paper size.
            - ``A4`` for a A4 paper size.
            - ``A5`` for a A5 paper size.
            - ``A6`` for a A6 paper size.
            - ``A7`` for a A7 paper size.
            - ``A8`` for a A8 paper size.
            - ``A9`` for a A9 paper size.
            - ``B0`` for a B0 paper size.
            - ``B1`` for a B1 paper size.
            - ``B2`` for a B2 paper size.
            - ``B3`` for a B3 paper size.
            - ``B4`` for a B4 paper size.
            - ``B5`` for a B5 paper size.
            - ``B6`` for a B6 paper size.
            - ``B7`` for a B7 paper size.
            - ``B8`` for a B8 paper size.
            - ``B9`` for a B9 paper size.
            - ``ROLL89MM`` for a 89 MM Roll paper size.
            - ``ROLL127MM`` for a 127 MM Roll paper size.
            - ``ROLL100MM`` for a 100 MM Roll paper size.
            - ``ROLL210MM`` for a 210 MM Roll paper size.
        """  # noqa: E501
        return self._papersize

    @property
    def papertype(self) -> PictbridgePapertype:
        """Return the ``PICTBridge:PAPERType`` command.

        **Description:**
            - Sets or returns the paper type.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``PICTBridge:PAPERType value``
              command.

        **SCPI Syntax:**

        ::

            - PICTBridge:PAPERType {DEFLT|PLAIN|PHOTO|FASTPHOTO}

        **Info:**
            - ``DEFLT`` for a default print paper type.
            - ``PLAIN`` for a plain print paper type.
            - ``PHOTO`` for a photo print paper type.
            - ``FASTPHOTO`` for a fastphoto print paper type.
        """
        return self._papertype

    @property
    def printqual(self) -> PictbridgePrintqual:
        """Return the ``PICTBridge:PRINTQual`` command.

        **Description:**
            - Sets or returns the output print quality.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``PICTBridge:PRINTQual value``
              command.

        **SCPI Syntax:**

        ::

            - PICTBridge:PRINTQual {DEFLT|NRMAL|FINE|DRAFT}

        **Info:**
            - ``DEFLT`` for the default quality print.
            - ``NRMAL`` for a normal quality print.
            - ``FINE`` for a fine quality print.
            - ``DRAFT`` for a draft quality print.
        """
        return self._printqual



"""DAQ6510 device driver module."""
from functools import cached_property
from typing import Tuple

import pyvisa as visa

from tm_devices.commands import DAQ6510Mixin
from tm_devices.drivers.pi._ieee488_2_commands import LegacyTSPIEEE4882Commands
from tm_devices.drivers.pi.data_acquisition_systems.data_acquisition_system import (
    DataAcquisitionSystem,
)
from tm_devices.helpers import DeviceConfigEntry


class DAQ6510(DAQ6510Mixin, DataAcquisitionSystem):
    """DAQ6510 device driver."""

    _IEEE_COMMANDS_CLASS = LegacyTSPIEEE4882Commands

    ################################################################################################
    # Magic Methods
    ################################################################################################
    def __init__(
        self,
        config_entry: DeviceConfigEntry,
        verbose: bool,
        visa_resource: visa.resources.MessageBasedResource,
    ) -> None:
        """Create a DAQ6510 device.

        Args:
            config_entry: A config entry object parsed by the DMConfigParser.
            verbose: A boolean indicating if verbose output should be printed.
            visa_resource: The VISA resource object.
        """
        super().__init__(config_entry, verbose, visa_resource)

    ################################################################################################
    # Properties
    ################################################################################################
    @property
    def all_channel_names_list(self) -> Tuple[str, ...]:
        """Return a tuple containing all the channel names."""
        return tuple(f"{x+1}" for x in range(self.total_channels))

    @property
    def ieee_cmds(self) -> LegacyTSPIEEE4882Commands:
        """Return an internal class containing methods for the standard IEEE 488.2 command set."""
        return self._ieee_cmds  # type: ignore

    @cached_property
    def total_channels(self) -> int:
        """Return the total number of channels (all types)."""
        return 1

    ################################################################################################
    # Public Methods
    ################################################################################################
    def get_eventlog_status(self) -> Tuple[bool, str]:
        """Help function for getting the eventlog status.

        Returns:
            Boolean indicating no error, String containing concatenated contents of event log.
        """
        result_allev = False
        allev_result_str = '0,"No events to report - queue empty"'

        if not (err_count := int(self.query("print(eventlog.getcount(eventlog.SEV_ERROR))"))):
            result_allev = True
        else:
            allev_result_list = [self.query("print(eventlog.next())") for _ in range(err_count)]
            allev_result_str = ",".join(allev_result_list)
        return result_allev, allev_result_str

    ################################################################################################
    # Private Methods
    ################################################################################################



"""The touchscreen commands module.

These commands are used in the following models:
LPD6, MSO4, MSO5, MSO5B, MSO5LP, MSO6, MSO6B

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - TOUCHSCReen:CALibrate STARt
    - TOUCHSCReen:STATe {0|1|OFF|ON}
    - TOUCHSCReen:STATe?
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class TouchscreenState(SCPICmdWrite, SCPICmdRead):
    """The ``TOUCHSCReen:STATe`` command.

    **Description:**
        - This sets or queries the enabled state of the touch screen only. This command is
          equivalent to pushing the Touch Off button on the front panel. To completely disable front
          panel operation, use the command LOCK ALL. To re-enable the front panel, send the command
          LOCK NONE.

    **Usage:**
        - Using the ``.query()`` method will send the ``TOUCHSCReen:STATe?`` query.
        - Using the ``.verify(value)`` method will send the ``TOUCHSCReen:STATe?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``TOUCHSCReen:STATe value`` command.

    **SCPI Syntax:**

    ::

        - TOUCHSCReen:STATe {0|1|OFF|ON}
        - TOUCHSCReen:STATe?

    **Info:**
        - ``0`` disables the touch screen.
        - ``ON`` enables the touch screen.
        - ``OFF`` disables the touch screen.
    """


class TouchscreenCalibrate(SCPICmdWrite):
    """The ``TOUCHSCReen:CALibrate`` command.

    **Description:**
        - This command launches the touch screen calibration procedure. This command is equivalent
          to tapping the Calibrate touch screen control in the Utility->Self Test menu.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``TOUCHSCReen:CALibrate value`` command.

    **SCPI Syntax:**

    ::

        - TOUCHSCReen:CALibrate STARt

    **Info:**
        - ``STARt`` launches the touch screen calibration procedure.
    """


class Touchscreen(SCPICmdRead):
    """The ``TOUCHSCReen`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``TOUCHSCReen?`` query.
        - Using the ``.verify(value)`` method will send the ``TOUCHSCReen?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.calibrate``: The ``TOUCHSCReen:CALibrate`` command.
        - ``.state``: The ``TOUCHSCReen:STATe`` command.
    """

    def __init__(
        self, device: Optional["PIDevice"] = None, cmd_syntax: str = "TOUCHSCReen"
    ) -> None:
        super().__init__(device, cmd_syntax)
        self._calibrate = TouchscreenCalibrate(device, f"{self._cmd_syntax}:CALibrate")
        self._state = TouchscreenState(device, f"{self._cmd_syntax}:STATe")

    @property
    def calibrate(self) -> TouchscreenCalibrate:
        """Return the ``TOUCHSCReen:CALibrate`` command.

        **Description:**
            - This command launches the touch screen calibration procedure. This command is
              equivalent to tapping the Calibrate touch screen control in the Utility->Self Test
              menu.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``TOUCHSCReen:CALibrate value``
              command.

        **SCPI Syntax:**

        ::

            - TOUCHSCReen:CALibrate STARt

        **Info:**
            - ``STARt`` launches the touch screen calibration procedure.
        """
        return self._calibrate

    @property
    def state(self) -> TouchscreenState:
        """Return the ``TOUCHSCReen:STATe`` command.

        **Description:**
            - This sets or queries the enabled state of the touch screen only. This command is
              equivalent to pushing the Touch Off button on the front panel. To completely disable
              front panel operation, use the command LOCK ALL. To re-enable the front panel, send
              the command LOCK NONE.

        **Usage:**
            - Using the ``.query()`` method will send the ``TOUCHSCReen:STATe?`` query.
            - Using the ``.verify(value)`` method will send the ``TOUCHSCReen:STATe?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``TOUCHSCReen:STATe value`` command.

        **SCPI Syntax:**

        ::

            - TOUCHSCReen:STATe {0|1|OFF|ON}
            - TOUCHSCReen:STATe?

        **Info:**
            - ``0`` disables the touch screen.
            - ``ON`` enables the touch screen.
            - ``OFF`` disables the touch screen.
        """
        return self._state



# ruff: noqa: D402,PLR0913
# pyright: reportConstantRedefinition=none
"""The display commands module.

These commands are used in the following models:
DMM7510

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:

::

    - display.activebuffer
    - display.changescreen()
    - display.clear()
    - display.delete()
    - display.input.number()
    - display.input.option()
    - display.input.prompt()
    - display.input.string()
    - display.lightstate
    - display.prompt()
    - display.readingformat
    - display.settext()
    - display.waitevent()
"""
from typing import Optional, TYPE_CHECKING, Union

from .._helpers import BaseTSPCmd, NoDeviceProvidedError

if TYPE_CHECKING:
    from tm_devices.drivers.pi.tsp_device import TSPDevice


class DisplayInput(BaseTSPCmd):
    """The ``display.input`` command tree.

    Properties/methods:
        - ``.number()``: The ``display.input.number()`` function.
        - ``.option()``: The ``display.input.option()`` function.
        - ``.prompt()``: The ``display.input.prompt()`` function.
        - ``.string()``: The ``display.input.string()`` function.
    """

    def number(
        self,
        dialog_title: str,
        number_format: Optional[str] = None,
        default_value: Optional[str] = None,
        minimum_value: Optional[str] = None,
        maximum_value: Optional[str] = None,
    ) -> str:
        """Run the ``display.input.number()`` function.

        **Description:**
            - This function allows you to create a prompt that requests a number from the user on
              the front-panel display.

        **TSP Syntax:**

        ::

            - display.input.number()

        Args:
            dialog_title: A string that contains the text to be displayed as the title of the dialog
                box on the front-panel display; can be up to 32 characters.
            number_format (optional): The format of the displayed number.
            default_value (optional): The value that is initially displayed in the displayed keypad.
            minimum_value (optional): The lowest value that can be entered.
            maximum_value (optional): The highest value that can be entered.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(
                str(x)
                for x in (
                    f'"{dialog_title}"',
                    number_format,
                    default_value,
                    minimum_value,
                    maximum_value,
                )
                if x is not None
            )
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.number({function_args}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.number()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def option(self, dialog_title: str, button_title1: str, button_title2: str) -> str:
        """Run the ``display.input.option()`` function.

        **Description:**
            - This function allows you to create an option dialog box with customizable buttons on
              the front-panel display.

        **TSP Syntax:**

        ::

            - display.input.option()

        Args:
            dialog_title: A string that contains the text to be displayed as the title of the dialog
                box on the front-panel display; up to 32 characters.
            button_title1: A string that contains the name of the first button; up to 15 characters.
            button_title2: A string that contains the name of the second button; up to 15
                characters.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f'print({self._cmd_syntax}.option("{dialog_title}", '
                f'"{button_title1}", '
                f'"{button_title2}"))'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.option()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def prompt(self, button_set: str, dialog_title: str) -> str:
        """Run the ``display.input.prompt()`` function.

        **Description:**
            - This function allows you to create a prompt that accepts a user response from the
              front-panel display.

        **TSP Syntax:**

        ::

            - display.input.prompt()

        Args:
            button_set: The set of buttons to display.
            dialog_title: A string that contains the text to be displayed as the title of the dialog
                box on the front-panel display; up to 63 characters.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f'print({self._cmd_syntax}.prompt({button_set}, "{dialog_title}"))'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.prompt()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def string(self, dialog_title: str, text_format: Optional[str] = None) -> str:
        """Run the ``display.input.string()`` function.

        **Description:**
            - This function allows you to create a dialog box that requests text from the user
              through the front-panel display.

        **TSP Syntax:**

        ::

            - display.input.string()

        Args:
            dialog_title: A string that contains the text to be displayed as the title of the dialog
                box on the front-panel display; up to 32 characters.
            text_format (optional): The format of the entered text.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(
                str(x)
                for x in (
                    f'"{dialog_title}"',
                    text_format,
                )
                if x is not None
            )
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.string({function_args}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.string()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class Display(BaseTSPCmd):
    """The ``display`` command tree.

    Constants:
        - ``.BUTTONS_CANCEL``: Display CANCEL on dialog.
        - ``.BUTTONS_NONE``: Display nothing on dialog.
        - ``.BUTTONS_OK``: Display OK on dialog.
        - ``.BUTTONS_OKCANCEL``: Display OK and CANCEL on dialog.
        - ``.BUTTONS_YESNO``: Display YES and NO on dialog.
        - ``.BUTTONS_YESNOCANCEL``: Display YES, NO, and CANCEL on dialog.
        - ``.BUTTON_CANCEL``: Return if CANCEL selected.
        - ``.BUTTON_NO``: Return if NO selected.
        - ``.BUTTON_OK``: Return if OK selected.
        - ``.BUTTON_YES``: Return if YES selected.
        - ``.FORMAT_EXPONENT``: Use exponent format to display measurement readings on the
          front-panel display of
          the instrument.
        - ``.FORMAT_PREFIX``: Add a prefix to the units symbol, such as k, m, or u, to display
          measurement readings on the front-panel display of
          the instrument.
        - ``.NFORMAT_DECIMAL``: Allow decimal values.
        - ``.NFORMAT_EXPONENT``: Display numbers in exponent format.
        - ``.NFORMAT_INTEGER``: Allow integers (negative or positive) only.
        - ``.NFORMAT_PREFIX``: Display numbers with prefixes before the units symbol, such as n, m,
          or u.
        - ``.SCREEN_GRAPH``: Display the last selected graph screen tab.
        - ``.SCREEN_GRAPH_SWIPE``: Display graph swipe screen.
        - ``.SCREEN_HISTOGRAM``: Display Histogram.
        - ``.SCREEN_HOME``: Display Home screen.
        - ``.SCREEN_HOME_LARGE_READING``: Display Home screen with large readings.
        - ``.SCREEN_PROCESSING``: Open a screen that uses minimal CPU resources.
        - ``.SCREEN_READING_TABLE``: Display Reading Table screen.
        - ``.SCREEN_SETTINGS_SWIPE``: Display SETTINGS swipe screen.
        - ``.SCREEN_STATS_SWIPE``: Display STATISTICS swipe screen.
        - ``.SCREEN_USER_SWIPE``: Display USER swipe screen.
        - ``.SFORMAT_ANY``: Allow any characters.
        - ``.SFORMAT_BUFFER_NAME``: Allow both upper- and lower-case letters (no special
          characters).
        - ``.SFORMAT_UPPER``: Allow only upper-case letters.
        - ``.SFORMAT_UPPER_LOWER``: Allow both upper- and lower-case letters (no special
          characters).
        - ``.STATE_BLACKOUT``: Set display and all indicators off.
        - ``.STATE_LCD_100``: Set display to full brightness.
        - ``.STATE_LCD_25``: Set display to 25% brightness.
        - ``.STATE_LCD_50``: Set display to 50% brightness.
        - ``.STATE_LCD_75``: Set display to 75% brightness.
        - ``.STATE_LCD_OFF``: Set display to off.

    Properties/methods:
        - ``.activebuffer``: The ``display.activebuffer`` attribute.
        - ``.changescreen()``: The ``display.changescreen()`` function.
        - ``.clear()``: The ``display.clear()`` function.
        - ``.delete()``: The ``display.delete()`` function.
        - ``.input``: The ``display.input`` command tree.
        - ``.lightstate``: The ``display.lightstate`` attribute.
        - ``.prompt()``: The ``display.prompt()`` function.
        - ``.readingformat``: The ``display.readingformat`` attribute.
        - ``.settext()``: The ``display.settext()`` function.
        - ``.waitevent()``: The ``display.waitevent()`` function.
    """

    BUTTONS_CANCEL = "display.BUTTONS_CANCEL"
    """str: Display CANCEL on dialog."""
    BUTTONS_NONE = "display.BUTTONS_NONE"
    """str: Display nothing on dialog."""
    BUTTONS_OK = "display.BUTTONS_OK"
    """str: Display OK on dialog."""
    BUTTONS_OKCANCEL = "display.BUTTONS_OKCANCEL"
    """str: Display OK and CANCEL on dialog."""
    BUTTONS_YESNO = "display.BUTTONS_YESNO"
    """str: Display YES and NO on dialog."""
    BUTTONS_YESNOCANCEL = "display.BUTTONS_YESNOCANCEL"
    """str: Display YES, NO, and CANCEL on dialog."""
    BUTTON_CANCEL = "display.BUTTON_CANCEL"
    """str: Return if CANCEL selected."""
    BUTTON_NO = "display.BUTTON_NO"
    """str: Return if NO selected."""
    BUTTON_OK = "display.BUTTON_OK"
    """str: Return if OK selected."""
    BUTTON_YES = "display.BUTTON_YES"
    """str: Return if YES selected."""
    FORMAT_EXPONENT = "display.FORMAT_EXPONENT"
    """str: Use exponent format to display measurement readings on the front-panel display of
the instrument."""
    FORMAT_PREFIX = "display.FORMAT_PREFIX"
    """str: Add a prefix to the units symbol, such as k, m, or u, to display measurement readings on the front-panel display of
the instrument."""  # noqa: E501
    NFORMAT_DECIMAL = "display.NFORMAT_DECIMAL"
    """str: Allow decimal values."""
    NFORMAT_EXPONENT = "display.NFORMAT_EXPONENT"
    """str: Display numbers in exponent format."""
    NFORMAT_INTEGER = "display.NFORMAT_INTEGER"
    """str: Allow integers (negative or positive) only."""
    NFORMAT_PREFIX = "display.NFORMAT_PREFIX"
    """str: Display numbers with prefixes before the units symbol, such as n, m, or u."""
    SCREEN_GRAPH = "display.SCREEN_GRAPH"
    """str: Display the last selected graph screen tab."""
    SCREEN_GRAPH_SWIPE = "display.SCREEN_GRAPH_SWIPE"
    """str: Display graph swipe screen."""
    SCREEN_HISTOGRAM = "display.SCREEN_HISTOGRAM"
    """str: Display Histogram."""
    SCREEN_HOME = "display.SCREEN_HOME"
    """str: Display Home screen."""
    SCREEN_HOME_LARGE_READING = "display.SCREEN_HOME_LARGE_READING"
    """str: Display Home screen with large readings."""
    SCREEN_PROCESSING = "display.SCREEN_PROCESSING"
    """str: Open a screen that uses minimal CPU resources."""
    SCREEN_READING_TABLE = "display.SCREEN_READING_TABLE"
    """str: Display Reading Table screen."""
    SCREEN_SETTINGS_SWIPE = "display.SCREEN_SETTINGS_SWIPE"
    """str: Display SETTINGS swipe screen."""
    SCREEN_STATS_SWIPE = "display.SCREEN_STATS_SWIPE"
    """str: Display STATISTICS swipe screen."""
    SCREEN_USER_SWIPE = "display.SCREEN_USER_SWIPE"
    """str: Display USER swipe screen."""
    SFORMAT_ANY = "display.SFORMAT_ANY"
    """str: Allow any characters."""
    SFORMAT_BUFFER_NAME = "display.SFORMAT_BUFFER_NAME"
    """str: Allow both upper- and lower-case letters (no special characters)."""
    SFORMAT_UPPER = "display.SFORMAT_UPPER"
    """str: Allow only upper-case letters."""
    SFORMAT_UPPER_LOWER = "display.SFORMAT_UPPER_LOWER"
    """str: Allow both upper- and lower-case letters (no special characters)."""
    STATE_BLACKOUT = "display.STATE_BLACKOUT"
    """str: Set display and all indicators off."""
    STATE_LCD_100 = "display.STATE_LCD_100"
    """str: Set display to full brightness."""
    STATE_LCD_25 = "display.STATE_LCD_25"
    """str: Set display to 25% brightness."""
    STATE_LCD_50 = "display.STATE_LCD_50"
    """str: Set display to 50% brightness."""
    STATE_LCD_75 = "display.STATE_LCD_75"
    """str: Set display to 75% brightness."""
    STATE_LCD_OFF = "display.STATE_LCD_OFF"
    """str: Set display to off."""

    def __init__(self, device: Optional["TSPDevice"] = None, cmd_syntax: str = "display") -> None:
        super().__init__(device, cmd_syntax)
        self._input = DisplayInput(device, f"{self._cmd_syntax}.input")

    @property
    def activebuffer(self) -> str:
        """Access the ``display.activebuffer`` attribute.

        **Description:**
            - This attribute determines which buffer is used for measurements that are displayed on
              the front panel.

        **Usage:**
            - Accessing this property will send the ``print(display.activebuffer)`` query.
            - Setting this property to a value will send the ``display.activebuffer = value``
              command.

        **TSP Syntax:**

        ::

            - display.activebuffer = value
            - print(display.activebuffer)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".activebuffer"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.activebuffer)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.activebuffer`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @activebuffer.setter
    def activebuffer(self, value: Union[str, float]) -> None:
        """Access the ``display.activebuffer`` attribute.

        **Description:**
            - This attribute determines which buffer is used for measurements that are displayed on
              the front panel.

        **Usage:**
            - Accessing this property will send the ``print(display.activebuffer)`` query.
            - Setting this property to a value will send the ``display.activebuffer = value``
              command.

        **TSP Syntax:**

        ::

            - display.activebuffer = value
            - print(display.activebuffer)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".activebuffer", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.activebuffer = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.activebuffer`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def input(self) -> DisplayInput:
        """Return the ``display.input`` command tree.

        Sub-properties/methods:
            - ``.number()``: The ``display.input.number()`` function.
            - ``.option()``: The ``display.input.option()`` function.
            - ``.prompt()``: The ``display.input.prompt()`` function.
            - ``.string()``: The ``display.input.string()`` function.
        """
        return self._input

    @property
    def lightstate(self) -> str:
        """Access the ``display.lightstate`` attribute.

        **Description:**
            - This attribute sets the light output level of the front-panel display.

        **Usage:**
            - Accessing this property will send the ``print(display.lightstate)`` query.
            - Setting this property to a value will send the ``display.lightstate = value`` command.

        **TSP Syntax:**

        ::

            - display.lightstate = value
            - print(display.lightstate)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".lightstate"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.lightstate)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.lightstate`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @lightstate.setter
    def lightstate(self, value: Union[str, float]) -> None:
        """Access the ``display.lightstate`` attribute.

        **Description:**
            - This attribute sets the light output level of the front-panel display.

        **Usage:**
            - Accessing this property will send the ``print(display.lightstate)`` query.
            - Setting this property to a value will send the ``display.lightstate = value`` command.

        **TSP Syntax:**

        ::

            - display.lightstate = value
            - print(display.lightstate)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".lightstate", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.lightstate = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.lightstate`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def readingformat(self) -> str:
        """Access the ``display.readingformat`` attribute.

        **Description:**
            - This attribute determines the format that is used to display measurement readings on
              the front-panel display of the instrument.

        **Usage:**
            - Accessing this property will send the ``print(display.readingformat)`` query.
            - Setting this property to a value will send the ``display.readingformat = value``
              command.

        **TSP Syntax:**

        ::

            - display.readingformat = value
            - print(display.readingformat)

        **Info:**
            - ``format``, the use exponent format: display.FORMAT_EXPONENT
              Add a prefix to the units symbol, such as k, m, or µ: display.FORMAT_PREFIX.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".readingformat"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.readingformat)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.readingformat`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @readingformat.setter
    def readingformat(self, value: Union[str, float]) -> None:
        """Access the ``display.readingformat`` attribute.

        **Description:**
            - This attribute determines the format that is used to display measurement readings on
              the front-panel display of the instrument.

        **Usage:**
            - Accessing this property will send the ``print(display.readingformat)`` query.
            - Setting this property to a value will send the ``display.readingformat = value``
              command.

        **TSP Syntax:**

        ::

            - display.readingformat = value
            - print(display.readingformat)

        **Info:**
            - ``format``, the use exponent format: display.FORMAT_EXPONENT
              Add a prefix to the units symbol, such as k, m, or µ: display.FORMAT_PREFIX.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".readingformat", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.readingformat = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.readingformat`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def changescreen(self, screen_name: str) -> None:
        """Run the ``display.changescreen()`` function.

        **Description:**
            - This function changes which front-panel screen is displayed.

        **TSP Syntax:**

        ::

            - display.changescreen()

        Args:
            screen_name: The screen to display.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.changescreen({screen_name})"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.changescreen()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def clear(self) -> None:
        """Run the ``display.clear()`` function.

        **Description:**
            - This function clears the text from the front-panel USER swipe screen.

        **TSP Syntax:**

        ::

            - display.clear()

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(f"{self._cmd_syntax}.clear()")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.clear()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def delete(self, prompt_id: str) -> None:
        """Run the ``display.delete()`` function.

        **Description:**
            - This function allows you to remove a prompt on the front-panel display that was
              created with display.prompt().

        **TSP Syntax:**

        ::

            - display.delete()

        Args:
            prompt_id: The identifier defined by display.prompt().

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.delete({prompt_id})"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.delete()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def prompt(self, button_id: str, prompt_text: str) -> str:
        """Run the ``display.prompt()`` function.

        **Description:**
            - This function allows you to create an interactive dialog prompt that displays a custom
              message on the front-panel display.

        **TSP Syntax:**

        ::

            - display.prompt()

        Args:
            button_id: The type of prompt to display; choose one of the following options.
            prompt_text: A string that contains the text that is displayed above the prompts.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f'print({self._cmd_syntax}.prompt({button_id}, "{prompt_text}"))'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.prompt()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def settext(
        self, user_display_text1: Optional[str] = None, user_display_text2: Optional[str] = None
    ) -> None:
        """Run the ``display.settext()`` function.

        **Description:**
            - This function defines the text that is displayed on the front-panel USER swipe screen.

        **TSP Syntax:**

        ::

            - display.settext()

        Args:
            user_display_text1 (optional): String that contains the message for the top line of the
                USER swipe screen (up to 20 characters).
            user_display_text2 (optional): String that contains the message for the bottom line of
                the USER swipe screen (up to 32 characters).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(
                str(x)
                for x in (
                    f'"{user_display_text1}"' if user_display_text1 is not None else None,
                    f'"{user_display_text2}"' if user_display_text2 is not None else None,
                )
                if x is not None
            )
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.settext({function_args})"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.settext()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def waitevent(self, timeout: Optional[float] = None) -> str:
        """Run the ``display.waitevent()`` function.

        **Description:**
            - This function causes the instrument to wait for a user to respond to a prompt that was
              created with a prompt command.

        **TSP Syntax:**

        ::

            - display.waitevent()

        Args:
            timeout (optional): The amount of time to wait before timing out; time is 0 to 300 s,
                where the default of 0 waits indefinitely.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(str(x) for x in (timeout,) if x is not None)
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.waitevent({function_args}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.waitevent()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error



"""The application commands module.

These commands are used in the following models:
DPO4K, DPO4KB, MDO3, MDO3K, MDO4K, MDO4KB, MDO4KC, MSO4K, MSO4KB

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - APPLication:LICENSE:SLOT<x>:LOCation?
    - APPLication:LICENSE:SLOT<x>:TRANSFER EXECute
    - APPLication:LICENSE:SLOT<x>:TYPe?
    - APPLication:TYPe {POWer|LIMITMask|VIDPic|ACTONEVent|NONe}
    - APPLication:TYPe?
"""
from typing import Dict, Optional, TYPE_CHECKING

from .._helpers import (
    DefaultDictPassKeyToFactory,
    SCPICmdRead,
    SCPICmdWrite,
    ValidatedDynamicNumberCmd,
)

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class ApplicationType(SCPICmdWrite, SCPICmdRead):
    """The ``APPLication:TYPe`` command.

    **Description:**
        - This command sets or returns the application type. The query form will return NONe if none
          of the supported test application modules are installed. Attempting to set the application
          type to a type with no application option installed will result in a settings conflict
          error event.

    **Usage:**
        - Using the ``.query()`` method will send the ``APPLication:TYPe?`` query.
        - Using the ``.verify(value)`` method will send the ``APPLication:TYPe?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``APPLication:TYPe value`` command.

    **SCPI Syntax:**

    ::

        - APPLication:TYPe {POWer|LIMITMask|VIDPic|ACTONEVent|NONe}
        - APPLication:TYPe?

    **Info:**
        - ``POWer`` sets the application type to power analysis.
        - ``LIMITMask`` sets the application type to limit mask test.
        - ``VIDPic`` sets the application type to video picture.
        - ``ACTONEVent`` sets the application type to act on event.
        - ``NONe``
    """


class ApplicationLicenseSlotItemType(SCPICmdRead):
    """The ``APPLication:LICENSE:SLOT<x>:TYPe`` command.

    **Description:**
        - This query returns the application license type of the option that is currently inserted
          in the specified application option slot. If there is no application option in the slot,
          NONE is returned. < x> can be slot number 1-4 (or 1-2 for 3 Series MDO models).

    **Usage:**
        - Using the ``.query()`` method will send the ``APPLication:LICENSE:SLOT<x>:TYPe?`` query.
        - Using the ``.verify(value)`` method will send the ``APPLication:LICENSE:SLOT<x>:TYPe?``
          query and raise an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - APPLication:LICENSE:SLOT<x>:TYPe?
    """


class ApplicationLicenseSlotItemTransfer(SCPICmdWrite):
    """The ``APPLication:LICENSE:SLOT<x>:TRANSFER`` command.

    **Description:**
        - You can use this command to transfer a license from a physical application option to an
          internal memory location within the oscilloscope, and to transfer it back. Once a license
          has been transferred to an internal location, the application that it enables can be used
          without the physical application option being present; the physical application option can
          be removed, thus freeing up a slot. However, the license then needs to be transferred back
          to the physical application option in order to use the license with another instrument.
          After licenses have been transferred, the oscilloscope power must be cycled in order to
          enable/disable the features affected by the option. Applications modules must only be
          installed and removed when the oscilloscope power is off. < x> can be slot number 1-4 (1-2
          for 3 Series MDO models).

    **Usage:**
        - Using the ``.write(value)`` method will send the
          ``APPLication:LICENSE:SLOT<x>:TRANSFER value`` command.

    **SCPI Syntax:**

    ::

        - APPLication:LICENSE:SLOT<x>:TRANSFER EXECute
    """


class ApplicationLicenseSlotItemLocation(SCPICmdRead):
    """The ``APPLication:LICENSE:SLOT<x>:LOCation`` command.

    **Description:**
        - This query returns the license location. < x> can be slot number 1-4 (1-2 for 3 Series MDO
          models).

    **Usage:**
        - Using the ``.query()`` method will send the ``APPLication:LICENSE:SLOT<x>:LOCation?``
          query.
        - Using the ``.verify(value)`` method will send the
          ``APPLication:LICENSE:SLOT<x>:LOCation?`` query and raise an AssertionError if the
          returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - APPLication:LICENSE:SLOT<x>:LOCation?
    """


class ApplicationLicenseSlotItem(ValidatedDynamicNumberCmd, SCPICmdRead):
    """The ``APPLication:LICENSE:SLOT<x>`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``APPLication:LICENSE:SLOT<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``APPLication:LICENSE:SLOT<x>?`` query
          and raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.location``: The ``APPLication:LICENSE:SLOT<x>:LOCation`` command.
        - ``.transfer``: The ``APPLication:LICENSE:SLOT<x>:TRANSFER`` command.
        - ``.type``: The ``APPLication:LICENSE:SLOT<x>:TYPe`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._location = ApplicationLicenseSlotItemLocation(device, f"{self._cmd_syntax}:LOCation")
        self._transfer = ApplicationLicenseSlotItemTransfer(device, f"{self._cmd_syntax}:TRANSFER")
        self._type = ApplicationLicenseSlotItemType(device, f"{self._cmd_syntax}:TYPe")

    @property
    def location(self) -> ApplicationLicenseSlotItemLocation:
        """Return the ``APPLication:LICENSE:SLOT<x>:LOCation`` command.

        **Description:**
            - This query returns the license location. < x> can be slot number 1-4 (1-2 for 3 Series
              MDO models).

        **Usage:**
            - Using the ``.query()`` method will send the ``APPLication:LICENSE:SLOT<x>:LOCation?``
              query.
            - Using the ``.verify(value)`` method will send the
              ``APPLication:LICENSE:SLOT<x>:LOCation?`` query and raise an AssertionError if the
              returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - APPLication:LICENSE:SLOT<x>:LOCation?
        """
        return self._location

    @property
    def transfer(self) -> ApplicationLicenseSlotItemTransfer:
        """Return the ``APPLication:LICENSE:SLOT<x>:TRANSFER`` command.

        **Description:**
            - You can use this command to transfer a license from a physical application option to
              an internal memory location within the oscilloscope, and to transfer it back. Once a
              license has been transferred to an internal location, the application that it enables
              can be used without the physical application option being present; the physical
              application option can be removed, thus freeing up a slot. However, the license then
              needs to be transferred back to the physical application option in order to use the
              license with another instrument. After licenses have been transferred, the
              oscilloscope power must be cycled in order to enable/disable the features affected by
              the option. Applications modules must only be installed and removed when the
              oscilloscope power is off. < x> can be slot number 1-4 (1-2 for 3 Series MDO models).

        **Usage:**
            - Using the ``.write(value)`` method will send the
              ``APPLication:LICENSE:SLOT<x>:TRANSFER value`` command.

        **SCPI Syntax:**

        ::

            - APPLication:LICENSE:SLOT<x>:TRANSFER EXECute
        """
        return self._transfer

    @property
    def type(self) -> ApplicationLicenseSlotItemType:
        """Return the ``APPLication:LICENSE:SLOT<x>:TYPe`` command.

        **Description:**
            - This query returns the application license type of the option that is currently
              inserted in the specified application option slot. If there is no application option
              in the slot, NONE is returned. < x> can be slot number 1-4 (or 1-2 for 3 Series MDO
              models).

        **Usage:**
            - Using the ``.query()`` method will send the ``APPLication:LICENSE:SLOT<x>:TYPe?``
              query.
            - Using the ``.verify(value)`` method will send the
              ``APPLication:LICENSE:SLOT<x>:TYPe?`` query and raise an AssertionError if the
              returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - APPLication:LICENSE:SLOT<x>:TYPe?
        """
        return self._type


class ApplicationLicense(SCPICmdRead):
    """The ``APPLication:LICENSE`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``APPLication:LICENSE?`` query.
        - Using the ``.verify(value)`` method will send the ``APPLication:LICENSE?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.slot``: The ``APPLication:LICENSE:SLOT<x>`` command tree.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._slot: Dict[int, ApplicationLicenseSlotItem] = DefaultDictPassKeyToFactory(
            lambda x: ApplicationLicenseSlotItem(device, f"{self._cmd_syntax}:SLOT{x}")
        )

    @property
    def slot(self) -> Dict[int, ApplicationLicenseSlotItem]:
        """Return the ``APPLication:LICENSE:SLOT<x>`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``APPLication:LICENSE:SLOT<x>?`` query.
            - Using the ``.verify(value)`` method will send the ``APPLication:LICENSE:SLOT<x>?``
              query and raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.location``: The ``APPLication:LICENSE:SLOT<x>:LOCation`` command.
            - ``.transfer``: The ``APPLication:LICENSE:SLOT<x>:TRANSFER`` command.
            - ``.type``: The ``APPLication:LICENSE:SLOT<x>:TYPe`` command.
        """
        return self._slot


class Application(SCPICmdRead):
    """The ``APPLication`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``APPLication?`` query.
        - Using the ``.verify(value)`` method will send the ``APPLication?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.license``: The ``APPLication:LICENSE`` command tree.
        - ``.type``: The ``APPLication:TYPe`` command.
    """

    def __init__(
        self, device: Optional["PIDevice"] = None, cmd_syntax: str = "APPLication"
    ) -> None:
        super().__init__(device, cmd_syntax)
        self._license = ApplicationLicense(device, f"{self._cmd_syntax}:LICENSE")
        self._type = ApplicationType(device, f"{self._cmd_syntax}:TYPe")

    @property
    def license(self) -> ApplicationLicense:
        """Return the ``APPLication:LICENSE`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``APPLication:LICENSE?`` query.
            - Using the ``.verify(value)`` method will send the ``APPLication:LICENSE?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.slot``: The ``APPLication:LICENSE:SLOT<x>`` command tree.
        """
        return self._license

    @property
    def type(self) -> ApplicationType:
        """Return the ``APPLication:TYPe`` command.

        **Description:**
            - This command sets or returns the application type. The query form will return NONe if
              none of the supported test application modules are installed. Attempting to set the
              application type to a type with no application option installed will result in a
              settings conflict error event.

        **Usage:**
            - Using the ``.query()`` method will send the ``APPLication:TYPe?`` query.
            - Using the ``.verify(value)`` method will send the ``APPLication:TYPe?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``APPLication:TYPe value`` command.

        **SCPI Syntax:**

        ::

            - APPLication:TYPe {POWer|LIMITMask|VIDPic|ACTONEVent|NONe}
            - APPLication:TYPe?

        **Info:**
            - ``POWer`` sets the application type to power analysis.
            - ``LIMITMask`` sets the application type to limit mask test.
            - ``VIDPic`` sets the application type to video picture.
            - ``ACTONEVent`` sets the application type to act on event.
            - ``NONe``
        """
        return self._type



"""The dch commands module.

These commands are used in the following models:
MSO2

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - DCH<x>_D<x>:LABel:COLor <QString>
    - DCH<x>_D<x>:LABel:COLor?
    - DCH<x>_D<x>:LABel:FONT:BOLD {<NR1>|OFF|ON}
    - DCH<x>_D<x>:LABel:FONT:BOLD?
    - DCH<x>_D<x>:LABel:FONT:ITALic {<NR1>|OFF|ON}
    - DCH<x>_D<x>:LABel:FONT:ITALic?
    - DCH<x>_D<x>:LABel:FONT:SIZE <NR1>
    - DCH<x>_D<x>:LABel:FONT:SIZE?
    - DCH<x>_D<x>:LABel:FONT:TYPE <QString>
    - DCH<x>_D<x>:LABel:FONT:TYPE?
    - DCH<x>_D<x>:LABel:FONT:UNDERline {<NR1>|OFF|ON}
    - DCH<x>_D<x>:LABel:FONT:UNDERline?
    - DCH<x>_D<x>:LABel:NAMe <QString>
    - DCH<x>_D<x>:LABel:NAMe?
    - DCH<x>_D<x>:THReshold <NR3>
    - DCH<x>_D<x>:THReshold?
"""
from typing import Dict, Optional, TYPE_CHECKING

from .._helpers import (
    DefaultDictPassKeyToFactory,
    SCPICmdRead,
    SCPICmdWrite,
    ValidatedDigitalBit,
    ValidatedDynamicNumberCmd,
)

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class DchItemDigitalBitThreshold(SCPICmdWrite, SCPICmdRead):
    """The ``DCH<x>_D<x>:THReshold`` command.

    **Description:**
        - This command sets or queries the threshold level in volts for the specified digital
          channel. If the source channel doesn't exist, a hardware missing error event is set.

    **Usage:**
        - Using the ``.query()`` method will send the ``DCH<x>_D<x>:THReshold?`` query.
        - Using the ``.verify(value)`` method will send the ``DCH<x>_D<x>:THReshold?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DCH<x>_D<x>:THReshold value`` command.

    **SCPI Syntax:**

    ::

        - DCH<x>_D<x>:THReshold <NR3>
        - DCH<x>_D<x>:THReshold?

    **Info:**
        - ``DCH<x>`` specifies the digital channel. The supported value is 1.
        - ``D<x>`` specifies the digital bits. The supported values are 0 to 15. 0 to 7 have common
          a threshold and 8 to 15 have a common threshold.
        - ``<NR3>`` specifies the threshold level in volts for the specified digital channel. The
          minimum value is -20 V and the maximum value is 30.0 V.
    """


class DchItemDigitalBitLabelName(SCPICmdWrite, SCPICmdRead):
    """The ``DCH<x>_D<x>:LABel:NAMe`` command.

    **Description:**
        - This command sets or queries the label of the specified digital bit.

    **Usage:**
        - Using the ``.query()`` method will send the ``DCH<x>_D<x>:LABel:NAMe?`` query.
        - Using the ``.verify(value)`` method will send the ``DCH<x>_D<x>:LABel:NAMe?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DCH<x>_D<x>:LABel:NAMe value`` command.

    **SCPI Syntax:**

    ::

        - DCH<x>_D<x>:LABel:NAMe <QString>
        - DCH<x>_D<x>:LABel:NAMe?

    **Info:**
        - ``DCH<x>`` specifies the digital channel. The supported value is 1.
        - ``D<x>`` specifies the digital bits. specifies the digital bits. The supported values are
          0 to 15 or DALL (all digital bits).
        - ``<QString>`` is the label name.
    """

    _WRAP_ARG_WITH_QUOTES = True


class DchItemDigitalBitLabelFontUnderline(SCPICmdWrite, SCPICmdRead):
    """The ``DCH<x>_D<x>:LABel:FONT:UNDERline`` command.

    **Description:**
        - This command sets or queries the underline state of the label of the specified digital
          bit.

    **Usage:**
        - Using the ``.query()`` method will send the ``DCH<x>_D<x>:LABel:FONT:UNDERline?`` query.
        - Using the ``.verify(value)`` method will send the ``DCH<x>_D<x>:LABel:FONT:UNDERline?``
          query and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the
          ``DCH<x>_D<x>:LABel:FONT:UNDERline value`` command.

    **SCPI Syntax:**

    ::

        - DCH<x>_D<x>:LABel:FONT:UNDERline {<NR1>|OFF|ON}
        - DCH<x>_D<x>:LABel:FONT:UNDERline?

    **Info:**
        - ``DCH<x>`` specifies the digital channel. The supported value is 1.
        - ``D<x>`` specifies the digital bits. The supported values are 0 to 15.
        - ``<NR1>`` = 0 turns off underline font; any other value turns on italic font.
        - ``OFF`` turns off underline font.
        - ``ON`` turns on underline font.
    """


class DchItemDigitalBitLabelFontType(SCPICmdWrite, SCPICmdRead):
    """The ``DCH<x>_D<x>:LABel:FONT:TYPE`` command.

    **Description:**
        - This command sets or queries the font type of the label of the specified digital bit, such
          as Arial or Times New Roman.

    **Usage:**
        - Using the ``.query()`` method will send the ``DCH<x>_D<x>:LABel:FONT:TYPE?`` query.
        - Using the ``.verify(value)`` method will send the ``DCH<x>_D<x>:LABel:FONT:TYPE?`` query
          and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DCH<x>_D<x>:LABel:FONT:TYPE value``
          command.

    **SCPI Syntax:**

    ::

        - DCH<x>_D<x>:LABel:FONT:TYPE <QString>
        - DCH<x>_D<x>:LABel:FONT:TYPE?

    **Info:**
        - ``DCH<x>`` specifies the digital channel. The supported value is 1.
        - ``D<x>`` specifies the digital bits. The supported values are 0 to 15.
        - ``<QString>`` is the font type of the label.
    """

    _WRAP_ARG_WITH_QUOTES = True


class DchItemDigitalBitLabelFontSize(SCPICmdWrite, SCPICmdRead):
    """The ``DCH<x>_D<x>:LABel:FONT:SIZE`` command.

    **Description:**
        - This command sets or queries the font size of the label of the specified digital bit.

    **Usage:**
        - Using the ``.query()`` method will send the ``DCH<x>_D<x>:LABel:FONT:SIZE?`` query.
        - Using the ``.verify(value)`` method will send the ``DCH<x>_D<x>:LABel:FONT:SIZE?`` query
          and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DCH<x>_D<x>:LABel:FONT:SIZE value``
          command.

    **SCPI Syntax:**

    ::

        - DCH<x>_D<x>:LABel:FONT:SIZE <NR1>
        - DCH<x>_D<x>:LABel:FONT:SIZE?

    **Info:**
        - ``DCH<x>`` specifies the digital channel. The supported value is 1.
        - ``D<x>`` specifies the digital bits. The supported values are 0 to 15.
        - ``<NR1>`` specifies the font size in points.
    """


class DchItemDigitalBitLabelFontItalic(SCPICmdWrite, SCPICmdRead):
    """The ``DCH<x>_D<x>:LABel:FONT:ITALic`` command.

    **Description:**
        - This command sets or queries the italic state of the label of the specified digital bit.

    **Usage:**
        - Using the ``.query()`` method will send the ``DCH<x>_D<x>:LABel:FONT:ITALic?`` query.
        - Using the ``.verify(value)`` method will send the ``DCH<x>_D<x>:LABel:FONT:ITALic?`` query
          and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DCH<x>_D<x>:LABel:FONT:ITALic value``
          command.

    **SCPI Syntax:**

    ::

        - DCH<x>_D<x>:LABel:FONT:ITALic {<NR1>|OFF|ON}
        - DCH<x>_D<x>:LABel:FONT:ITALic?

    **Info:**
        - ``DCH<x>`` specifies the digital channel. The supported value is 1.
        - ``D<x>`` specifies the digital bits. The supported values are 0 to 15.
        - ``<NR1>`` = 0 turns off italic font; any other value turns on italic font.
        - ``OFF`` turns off italic font.
        - ``ON`` turns on italic font.
    """


class DchItemDigitalBitLabelFontBold(SCPICmdWrite, SCPICmdRead):
    """The ``DCH<x>_D<x>:LABel:FONT:BOLD`` command.

    **Description:**
        - This command sets or queries the bold state of the label of the specified digital bit.

    **Usage:**
        - Using the ``.query()`` method will send the ``DCH<x>_D<x>:LABel:FONT:BOLD?`` query.
        - Using the ``.verify(value)`` method will send the ``DCH<x>_D<x>:LABel:FONT:BOLD?`` query
          and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DCH<x>_D<x>:LABel:FONT:BOLD value``
          command.

    **SCPI Syntax:**

    ::

        - DCH<x>_D<x>:LABel:FONT:BOLD {<NR1>|OFF|ON}
        - DCH<x>_D<x>:LABel:FONT:BOLD?

    **Info:**
        - ``DCH<x>`` specifies the digital channel. The supported value is 1.
        - ``D<x>`` specifies the digital bits. The supported values are 0 to 15.
        - ``<NR1>`` = 0 turns off bold font; any other value turns on bold font.
        - ``OFF`` turns off bold font.
        - ``ON`` turns on bold font.
    """


class DchItemDigitalBitLabelFont(SCPICmdRead):
    """The ``DCH<x>_D<x>:LABel:FONT`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``DCH<x>_D<x>:LABel:FONT?`` query.
        - Using the ``.verify(value)`` method will send the ``DCH<x>_D<x>:LABel:FONT?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    **Info:**
        - ``DCH<x>`` specifies the digital channel. The supported value is 1.
        - ``D<x>`` specifies the digital bits. The supported values are 0 to 15.

    Properties:
        - ``.bold``: The ``DCH<x>_D<x>:LABel:FONT:BOLD`` command.
        - ``.italic``: The ``DCH<x>_D<x>:LABel:FONT:ITALic`` command.
        - ``.size``: The ``DCH<x>_D<x>:LABel:FONT:SIZE`` command.
        - ``.type``: The ``DCH<x>_D<x>:LABel:FONT:TYPE`` command.
        - ``.underline``: The ``DCH<x>_D<x>:LABel:FONT:UNDERline`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._bold = DchItemDigitalBitLabelFontBold(device, f"{self._cmd_syntax}:BOLD")
        self._italic = DchItemDigitalBitLabelFontItalic(device, f"{self._cmd_syntax}:ITALic")
        self._size = DchItemDigitalBitLabelFontSize(device, f"{self._cmd_syntax}:SIZE")
        self._type = DchItemDigitalBitLabelFontType(device, f"{self._cmd_syntax}:TYPE")
        self._underline = DchItemDigitalBitLabelFontUnderline(
            device, f"{self._cmd_syntax}:UNDERline"
        )

    @property
    def bold(self) -> DchItemDigitalBitLabelFontBold:
        """Return the ``DCH<x>_D<x>:LABel:FONT:BOLD`` command.

        **Description:**
            - This command sets or queries the bold state of the label of the specified digital bit.

        **Usage:**
            - Using the ``.query()`` method will send the ``DCH<x>_D<x>:LABel:FONT:BOLD?`` query.
            - Using the ``.verify(value)`` method will send the ``DCH<x>_D<x>:LABel:FONT:BOLD?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``DCH<x>_D<x>:LABel:FONT:BOLD value``
              command.

        **SCPI Syntax:**

        ::

            - DCH<x>_D<x>:LABel:FONT:BOLD {<NR1>|OFF|ON}
            - DCH<x>_D<x>:LABel:FONT:BOLD?

        **Info:**
            - ``DCH<x>`` specifies the digital channel. The supported value is 1.
            - ``D<x>`` specifies the digital bits. The supported values are 0 to 15.
            - ``<NR1>`` = 0 turns off bold font; any other value turns on bold font.
            - ``OFF`` turns off bold font.
            - ``ON`` turns on bold font.
        """
        return self._bold

    @property
    def italic(self) -> DchItemDigitalBitLabelFontItalic:
        """Return the ``DCH<x>_D<x>:LABel:FONT:ITALic`` command.

        **Description:**
            - This command sets or queries the italic state of the label of the specified digital
              bit.

        **Usage:**
            - Using the ``.query()`` method will send the ``DCH<x>_D<x>:LABel:FONT:ITALic?`` query.
            - Using the ``.verify(value)`` method will send the ``DCH<x>_D<x>:LABel:FONT:ITALic?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``DCH<x>_D<x>:LABel:FONT:ITALic value`` command.

        **SCPI Syntax:**

        ::

            - DCH<x>_D<x>:LABel:FONT:ITALic {<NR1>|OFF|ON}
            - DCH<x>_D<x>:LABel:FONT:ITALic?

        **Info:**
            - ``DCH<x>`` specifies the digital channel. The supported value is 1.
            - ``D<x>`` specifies the digital bits. The supported values are 0 to 15.
            - ``<NR1>`` = 0 turns off italic font; any other value turns on italic font.
            - ``OFF`` turns off italic font.
            - ``ON`` turns on italic font.
        """
        return self._italic

    @property
    def size(self) -> DchItemDigitalBitLabelFontSize:
        """Return the ``DCH<x>_D<x>:LABel:FONT:SIZE`` command.

        **Description:**
            - This command sets or queries the font size of the label of the specified digital bit.

        **Usage:**
            - Using the ``.query()`` method will send the ``DCH<x>_D<x>:LABel:FONT:SIZE?`` query.
            - Using the ``.verify(value)`` method will send the ``DCH<x>_D<x>:LABel:FONT:SIZE?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``DCH<x>_D<x>:LABel:FONT:SIZE value``
              command.

        **SCPI Syntax:**

        ::

            - DCH<x>_D<x>:LABel:FONT:SIZE <NR1>
            - DCH<x>_D<x>:LABel:FONT:SIZE?

        **Info:**
            - ``DCH<x>`` specifies the digital channel. The supported value is 1.
            - ``D<x>`` specifies the digital bits. The supported values are 0 to 15.
            - ``<NR1>`` specifies the font size in points.
        """
        return self._size

    @property
    def type(self) -> DchItemDigitalBitLabelFontType:
        """Return the ``DCH<x>_D<x>:LABel:FONT:TYPE`` command.

        **Description:**
            - This command sets or queries the font type of the label of the specified digital bit,
              such as Arial or Times New Roman.

        **Usage:**
            - Using the ``.query()`` method will send the ``DCH<x>_D<x>:LABel:FONT:TYPE?`` query.
            - Using the ``.verify(value)`` method will send the ``DCH<x>_D<x>:LABel:FONT:TYPE?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``DCH<x>_D<x>:LABel:FONT:TYPE value``
              command.

        **SCPI Syntax:**

        ::

            - DCH<x>_D<x>:LABel:FONT:TYPE <QString>
            - DCH<x>_D<x>:LABel:FONT:TYPE?

        **Info:**
            - ``DCH<x>`` specifies the digital channel. The supported value is 1.
            - ``D<x>`` specifies the digital bits. The supported values are 0 to 15.
            - ``<QString>`` is the font type of the label.
        """
        return self._type

    @property
    def underline(self) -> DchItemDigitalBitLabelFontUnderline:
        """Return the ``DCH<x>_D<x>:LABel:FONT:UNDERline`` command.

        **Description:**
            - This command sets or queries the underline state of the label of the specified digital
              bit.

        **Usage:**
            - Using the ``.query()`` method will send the ``DCH<x>_D<x>:LABel:FONT:UNDERline?``
              query.
            - Using the ``.verify(value)`` method will send the
              ``DCH<x>_D<x>:LABel:FONT:UNDERline?`` query and raise an AssertionError if the
              returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``DCH<x>_D<x>:LABel:FONT:UNDERline value`` command.

        **SCPI Syntax:**

        ::

            - DCH<x>_D<x>:LABel:FONT:UNDERline {<NR1>|OFF|ON}
            - DCH<x>_D<x>:LABel:FONT:UNDERline?

        **Info:**
            - ``DCH<x>`` specifies the digital channel. The supported value is 1.
            - ``D<x>`` specifies the digital bits. The supported values are 0 to 15.
            - ``<NR1>`` = 0 turns off underline font; any other value turns on italic font.
            - ``OFF`` turns off underline font.
            - ``ON`` turns on underline font.
        """
        return self._underline


class DchItemDigitalBitLabelColor(SCPICmdWrite, SCPICmdRead):
    """The ``DCH<x>_D<x>:LABel:COLor`` command.

    **Description:**
        - This command sets or queries the color of the label of the specified digital bit.

    **Usage:**
        - Using the ``.query()`` method will send the ``DCH<x>_D<x>:LABel:COLor?`` query.
        - Using the ``.verify(value)`` method will send the ``DCH<x>_D<x>:LABel:COLor?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DCH<x>_D<x>:LABel:COLor value``
          command.

    **SCPI Syntax:**

    ::

        - DCH<x>_D<x>:LABel:COLor <QString>
        - DCH<x>_D<x>:LABel:COLor?

    **Info:**
        - ``DCH<x>`` specifies the digital channel. The supported value is 1.
        - ``D<x>`` specifies the digital bits. The supported values are 0 to 15.
        - ``<QString>`` is the label color. To return the color to the default color, send an empty
          string as in this example: ``:DCH1_D1:LABEL:COLOR`` ''.
    """

    _WRAP_ARG_WITH_QUOTES = True


class DchItemDigitalBitLabel(SCPICmdRead):
    """The ``DCH<x>_D<x>:LABel`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``DCH<x>_D<x>:LABel?`` query.
        - Using the ``.verify(value)`` method will send the ``DCH<x>_D<x>:LABel?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    **Info:**
        - ``DCH<x>`` specifies the digital channel. The supported value is 1.
        - ``D<x>`` specifies the digital bits. The supported values are 0 to 15.

    Properties:
        - ``.color``: The ``DCH<x>_D<x>:LABel:COLor`` command.
        - ``.font``: The ``DCH<x>_D<x>:LABel:FONT`` command tree.
        - ``.name``: The ``DCH<x>_D<x>:LABel:NAMe`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._color = DchItemDigitalBitLabelColor(device, f"{self._cmd_syntax}:COLor")
        self._font = DchItemDigitalBitLabelFont(device, f"{self._cmd_syntax}:FONT")
        self._name = DchItemDigitalBitLabelName(device, f"{self._cmd_syntax}:NAMe")

    @property
    def color(self) -> DchItemDigitalBitLabelColor:
        """Return the ``DCH<x>_D<x>:LABel:COLor`` command.

        **Description:**
            - This command sets or queries the color of the label of the specified digital bit.

        **Usage:**
            - Using the ``.query()`` method will send the ``DCH<x>_D<x>:LABel:COLor?`` query.
            - Using the ``.verify(value)`` method will send the ``DCH<x>_D<x>:LABel:COLor?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``DCH<x>_D<x>:LABel:COLor value``
              command.

        **SCPI Syntax:**

        ::

            - DCH<x>_D<x>:LABel:COLor <QString>
            - DCH<x>_D<x>:LABel:COLor?

        **Info:**
            - ``DCH<x>`` specifies the digital channel. The supported value is 1.
            - ``D<x>`` specifies the digital bits. The supported values are 0 to 15.
            - ``<QString>`` is the label color. To return the color to the default color, send an
              empty string as in this example: ``:DCH1_D1:LABEL:COLOR`` ''.
        """
        return self._color

    @property
    def font(self) -> DchItemDigitalBitLabelFont:
        """Return the ``DCH<x>_D<x>:LABel:FONT`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``DCH<x>_D<x>:LABel:FONT?`` query.
            - Using the ``.verify(value)`` method will send the ``DCH<x>_D<x>:LABel:FONT?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        **Info:**
            - ``DCH<x>`` specifies the digital channel. The supported value is 1.
            - ``D<x>`` specifies the digital bits. The supported values are 0 to 15.

        Sub-properties:
            - ``.bold``: The ``DCH<x>_D<x>:LABel:FONT:BOLD`` command.
            - ``.italic``: The ``DCH<x>_D<x>:LABel:FONT:ITALic`` command.
            - ``.size``: The ``DCH<x>_D<x>:LABel:FONT:SIZE`` command.
            - ``.type``: The ``DCH<x>_D<x>:LABel:FONT:TYPE`` command.
            - ``.underline``: The ``DCH<x>_D<x>:LABel:FONT:UNDERline`` command.
        """
        return self._font

    @property
    def name(self) -> DchItemDigitalBitLabelName:
        """Return the ``DCH<x>_D<x>:LABel:NAMe`` command.

        **Description:**
            - This command sets or queries the label of the specified digital bit.

        **Usage:**
            - Using the ``.query()`` method will send the ``DCH<x>_D<x>:LABel:NAMe?`` query.
            - Using the ``.verify(value)`` method will send the ``DCH<x>_D<x>:LABel:NAMe?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``DCH<x>_D<x>:LABel:NAMe value``
              command.

        **SCPI Syntax:**

        ::

            - DCH<x>_D<x>:LABel:NAMe <QString>
            - DCH<x>_D<x>:LABel:NAMe?

        **Info:**
            - ``DCH<x>`` specifies the digital channel. The supported value is 1.
            - ``D<x>`` specifies the digital bits. specifies the digital bits. The supported values
              are 0 to 15 or DALL (all digital bits).
            - ``<QString>`` is the label name.
        """
        return self._name


class DchItemDigitalBit(ValidatedDigitalBit, SCPICmdRead):
    """The ``DCH<x>_D<x>`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``DCH<x>_D<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``DCH<x>_D<x>?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **Info:**
        - ``DCH<x>`` specifies the digital channel. The supported value is 1.
        - ``D<x>`` specifies the digital bits. The supported values are 0 to 15.

    Properties:
        - ``.label``: The ``DCH<x>_D<x>:LABel`` command tree.
        - ``.threshold``: The ``DCH<x>_D<x>:THReshold`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._label = DchItemDigitalBitLabel(device, f"{self._cmd_syntax}:LABel")
        self._threshold = DchItemDigitalBitThreshold(device, f"{self._cmd_syntax}:THReshold")

    @property
    def label(self) -> DchItemDigitalBitLabel:
        """Return the ``DCH<x>_D<x>:LABel`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``DCH<x>_D<x>:LABel?`` query.
            - Using the ``.verify(value)`` method will send the ``DCH<x>_D<x>:LABel?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        **Info:**
            - ``DCH<x>`` specifies the digital channel. The supported value is 1.
            - ``D<x>`` specifies the digital bits. The supported values are 0 to 15.

        Sub-properties:
            - ``.color``: The ``DCH<x>_D<x>:LABel:COLor`` command.
            - ``.font``: The ``DCH<x>_D<x>:LABel:FONT`` command tree.
            - ``.name``: The ``DCH<x>_D<x>:LABel:NAMe`` command.
        """
        return self._label

    @property
    def threshold(self) -> DchItemDigitalBitThreshold:
        """Return the ``DCH<x>_D<x>:THReshold`` command.

        **Description:**
            - This command sets or queries the threshold level in volts for the specified digital
              channel. If the source channel doesn't exist, a hardware missing error event is set.

        **Usage:**
            - Using the ``.query()`` method will send the ``DCH<x>_D<x>:THReshold?`` query.
            - Using the ``.verify(value)`` method will send the ``DCH<x>_D<x>:THReshold?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``DCH<x>_D<x>:THReshold value``
              command.

        **SCPI Syntax:**

        ::

            - DCH<x>_D<x>:THReshold <NR3>
            - DCH<x>_D<x>:THReshold?

        **Info:**
            - ``DCH<x>`` specifies the digital channel. The supported value is 1.
            - ``D<x>`` specifies the digital bits. The supported values are 0 to 15. 0 to 7 have
              common a threshold and 8 to 15 have a common threshold.
            - ``<NR3>`` specifies the threshold level in volts for the specified digital channel.
              The minimum value is -20 V and the maximum value is 30.0 V.
        """
        return self._threshold


class DchItem(ValidatedDynamicNumberCmd, SCPICmdRead):
    """The ``DCH<x>`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``DCH<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``DCH<x>?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **Info:**
        - ``DCH<x>`` specifies the digital channel. The supported value is 1.

    Properties:
        - ``.d``: The ``DCH<x>_D<x>`` command tree.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "DCH<x>") -> None:
        super().__init__(device, cmd_syntax)
        self._d: Dict[int, DchItemDigitalBit] = DefaultDictPassKeyToFactory(
            lambda x: DchItemDigitalBit(device, f"{self._cmd_syntax}_D{x}")
        )

    @property
    def d(self) -> Dict[int, DchItemDigitalBit]:
        """Return the ``DCH<x>_D<x>`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``DCH<x>_D<x>?`` query.
            - Using the ``.verify(value)`` method will send the ``DCH<x>_D<x>?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        **Info:**
            - ``DCH<x>`` specifies the digital channel. The supported value is 1.
            - ``D<x>`` specifies the digital bits. The supported values are 0 to 15.

        Sub-properties:
            - ``.label``: The ``DCH<x>_D<x>:LABel`` command tree.
            - ``.threshold``: The ``DCH<x>_D<x>:THReshold`` command.
        """
        return self._d



"""The linktraining commands module.

These commands are used in the following models:
DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO7KC, DSA70KC, DSA70KD, MSO5KB, MSO70KC, MSO70KDX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - LINKTRaining:ACQTime <NR1>
    - LINKTRaining:ACQTime?
    - LINKTRaining:ARMscope <OFF | ON>
    - LINKTRaining:ARMscope?
    - LINKTRaining:DECOde <OFF | ON>
    - LINKTRaining:DECOde?
    - LINKTRaining:EQUalizationCH<x> <NR1>
    - LINKTRaining:EQUalizationCH<x>?
    - LINKTRaining:LANE <x> {<NR1>|OFF|ON}
    - LINKTRaining:LANE?
    - LINKTRaining:MARK <FRAme | CONtrol | TRAining>
    - LINKTRaining:SETUP <SAVe |RESTore>
"""
from typing import Dict, Optional, TYPE_CHECKING

from .._helpers import (
    DefaultDictPassKeyToFactory,
    SCPICmdRead,
    SCPICmdWrite,
    ValidatedDynamicNumberCmd,
)

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class LinktrainingSetup(SCPICmdWrite):
    """The ``LINKTRaining:SETUP`` command.

    **Description:**
        - This 70KSX command saves or restores custom Link Training setup files. The setup files are
          complete instrument setup files. The setup files are saved/restored depending on the
          current values for Link Training Standard, Trigger-On, and Lane selections. You only need
          to use this command when you desire to preserve custom Vertical, Horizontal, or Zoom
          settings for the particular Link Training mode you are using. When Link Training is
          enabled, you may dismiss the pop-up control window and make custom changes to the
          Vertical, Horizontal, or Zoom settings, for example, to accommodate various probes or
          record length requirements. If you want to save these settings for future use, just issue
          the ``LINKTRaining:SETUP SAVe`` command. You can restore the factory Link Training setup
          at any time by using the ``LINKTRaining:SETUP RESTore`` command. UI buttons for these
          operations are also supplied via the Customize button on the pop-up UI for Link Training.
          You do not need to enter any file or directory names to use these commands. When you save
          your custom setup with this command, the Link Training setup file pertaining to the
          current Standard, Trigger-On, and Lane selections is overwritten with your new setup. You
          may restore the factory Link Training setup file at any time. Again, these commands
          save/restore the complete instrument setup for the particular Link Training mode currently
          selected. The Link Training application is only available on the 70KSX scopes.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``LINKTRaining:SETUP value`` command.

    **SCPI Syntax:**

    ::

        - LINKTRaining:SETUP <SAVe |RESTore>

    **Info:**
        - ``SAVe`` saves your custom Link Training setup.
        - ``RESTore`` restores the custom Link Training setup.
    """


class LinktrainingMark(SCPICmdWrite):
    """The ``LINKTRaining:MARK`` command.

    **Description:**
        - This command sets or queries the Mark setting of the Link Training application. The
          default argument value is FRAme, which marks the Frame Marker. When the argument is set to
          CONtrol, the frame Control Channel information is marked (32 cells of
          differential-Manchester-encoded request/response data). When the argument is TRAining, the
          PRBS Training Data is marked. Marking the PRBS Training Data is useful for giving you the
          beginning/ending of the Training Data section of the frame when you want to use
          cursor-gated DPOJET measurements. Link Training is only available on the 70KSX family of
          oscilloscopes.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``LINKTRaining:MARK value`` command.

    **SCPI Syntax:**

    ::

        - LINKTRaining:MARK <FRAme | CONtrol | TRAining>

    **Info:**
        - ``FRAme`` sets Link Training MARK to FRAME.
        - ``CONtrol`` sets Link Training MARK to CONTROL.
        - ``TRAining`` sets Link Training MARK to TRAINING.
    """


class LinktrainingLane(SCPICmdWrite, SCPICmdRead):
    """The ``LINKTRaining:LANE`` command.

    **Description:**
        - This command sets or queries the activation of the individual communications lanes under
          test. The defaults are Lane1 ON, and Lane2 OFF. It is taken as given that Lane1 uses
          oscilloscope Ch1 and Ch3, Lane2 uses oscilloscope Ch2 and Ch4 (to maximize sample rate).
          ATI DPO70000SX oscilloscopes only support Lane1 (on Ch1 and Ch3). <X> is the lane number
          and can be 1 or 2. One or two lanes may be tested on a single 4-channel DPO70000SX
          oscilloscope. To test up to 4 lanes simultaneously, two oscilloscopes running in
          Standalone mode may be used. This is necessary because the Time-Sync Standalone mode does
          not support independent triggering by multiple oscilloscopes. The associated Link Training
          Arm PI command or UI button, may be used to arm each oscilloscope individually. When the
          Link Training signal is generated by the devices under test, the individual oscilloscopes
          will each trigger, capturing the waveforms on their respective displays and the protocol
          information in their respective Results Tables. Note that each Lane requires two
          oscilloscope channels, because the lanes are bi-directional (Full Duplex). Link Training
          is available on DPO70000SX oscilloscopes only.

    **Usage:**
        - Using the ``.query()`` method will send the ``LINKTRaining:LANE?`` query.
        - Using the ``.verify(value)`` method will send the ``LINKTRaining:LANE?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``LINKTRaining:LANE value`` command.

    **SCPI Syntax:**

    ::

        - LINKTRaining:LANE <x> {<NR1>|OFF|ON}
        - LINKTRaining:LANE?

    **Info:**
        - ``<NR1>`` = 0 disables the specified lane; any other value enables the specified lane.
        - ``OFF`` disables the specified lane.
        - ``ON`` enables the specified lane.
    """


class LinktrainingEqualizationchItem(ValidatedDynamicNumberCmd, SCPICmdWrite, SCPICmdRead):
    """The ``LINKTRaining:EQUalizationCH<x>`` command.

    **Description:**
        - This 70KSX command sets the equalization value for Link Training of the specified channel.
          The default setting is zero. The units are dB. If you are probing a signal near to the
          transmitter, you likely do not need to increase the equalization value. However, if you
          are probing on a backplane, or other situation where the signal is seriously attenuated,
          you may want to increase the equalization value. Note that this equalization only applies
          to the trigger path in the oscilloscope. It does not apply to the data path. Use the SDLA
          option to provide equalization/de-embedding on the data path. Increasing the trigger path
          equalization value will improve the protocol data recovered into the Link Training Results
          Table when you are probing a seriously attenuated signal. Use the SDLA option to improve
          the equalization of the data path waveforms displayed on the oscilloscope. Since the
          protocol data (Ethernet Control Channel Coefficient Updates and Status Responses) is
          transmitted at ¼ the nominal bit rate (25 Gb/s for 100G Ethernet), the equalization value
          should be set considering ½ that reduced bitrate. For example, for 100G Ethernet, the
          protocol data is transmitted at 6.25 Gb/s, so set the equalization value considering half
          of that rate, 3.125 Gb/s. The Link Training application is only available on the 70KSX
          scopes.

    **Usage:**
        - Using the ``.query()`` method will send the ``LINKTRaining:EQUalizationCH<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``LINKTRaining:EQUalizationCH<x>?``
          query and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``LINKTRaining:EQUalizationCH<x> value``
          command.

    **SCPI Syntax:**

    ::

        - LINKTRaining:EQUalizationCH<x> <NR1>
        - LINKTRaining:EQUalizationCH<x>?
    """


class LinktrainingDecode(SCPICmdWrite, SCPICmdRead):
    """The ``LINKTRaining:DECOde`` command.

    **Description:**
        - This 70KSX command sets or queries the Decode setting of the Link Training application.
          The default argument value is ON, which decodes the Control Channel information on the
          waveform display. Other system decode parameters optionally allow the Control Channel Hex
          Decode to be expanded to also include bit-level decode. The only plausible reason to turn
          off decoding is to save time when you are capturing a very long record using TRIGgeron
          ``FIRst_frame`` or ``LASt_frame``. With TRIGgeron ``ALL_frames``, the overhead for
          decoding is very small. Link Training is only available on the 70KSX family of
          oscilloscopes.

    **Usage:**
        - Using the ``.query()`` method will send the ``LINKTRaining:DECOde?`` query.
        - Using the ``.verify(value)`` method will send the ``LINKTRaining:DECOde?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``LINKTRaining:DECOde value`` command.

    **SCPI Syntax:**

    ::

        - LINKTRaining:DECOde <OFF | ON>
        - LINKTRaining:DECOde?

    **Info:**
        - ``OFF`` indicates decode is OFF.
        - ``ON`` indicates decode is ON.
    """


class LinktrainingArmscope(SCPICmdWrite, SCPICmdRead):
    """The ``LINKTRaining:ARMscope`` command.

    **Description:**
        - This 70KSX command sets or queries the ARMscope setting of the Link Training application
          on the 70KSX family of scopes. The default argument value is OFF. To use the Link Training
          application, the State argument must first be set to ON. With the DUT Lane 'powered down'
          (flat-line), you then Arm the scope for single-step capture using this command. When you
          subsequently initiate Link Training on the DUT ('power-up' the lane), the Control Channel
          data and digitized waveforms are captured by the scope. At a minimum, to capture the Link
          Training data into a Results Table, both the ``LINKTTRaining:STAte ON`` and
          ``LINKTRaining:ARMscope ON`` commands must be issued, in that order. When Link Training is
          initiated on the DUT lane, the Results Table data and digitized waveforms will be
          captured. You may select other Link Training settings using the related commands shown
          below to control those features. Initiating Link Training on the DUT is a DUT-dependent
          operation that is usually accomplished by a (serial or other bus) command to the DUT to
          power-up the Lane under test. Sometimes, it can be as simple as un-plugging and
          re-plugging the signal connector for the lane. Note that it is not necessary to set the
          ARMscope to OFF as this is done automatically by the system. To summarize, the minimum
          sequence of events to capture 100G Ethernet KR4 from Default Setup is
          ``:LINKTRaining:STATE ON`` ``:LINKTRaining:ARMscope ON`` Initiate Link Training on the
          Lane under test on the DUT. The Results Table and Waveforms are captured on the scope. To
          repeat the experiment, all you have to do is power-down the lane on the DUT, re-issue the
          ARMscope command, and power-up the lane on the DUT again. Link Training is only available
          on the 70KSX oscilloscopes.

    **Usage:**
        - Using the ``.query()`` method will send the ``LINKTRaining:ARMscope?`` query.
        - Using the ``.verify(value)`` method will send the ``LINKTRaining:ARMscope?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``LINKTRaining:ARMscope value`` command.

    **SCPI Syntax:**

    ::

        - LINKTRaining:ARMscope <OFF | ON>
        - LINKTRaining:ARMscope?

    **Info:**
        - ``OFF`` indicates ARMscope is OFF.
        - ``ON`` indicates ARMscope is ON.
    """


class LinktrainingAcqtime(SCPICmdWrite, SCPICmdRead):
    """The ``LINKTRaining:ACQTime`` command.

    **Description:**
        - This 70KSX command sets the maximum Acquire Time for Link Training from 1 to 10 seconds.
          The default value is 2 seconds. The link training process is defined by the IEEE to
          complete within 500 ms. So, the default 2 seconds is sufficient for most purposes.
          However, some devices can be put into a mode where the Link Training process is repeated
          continuously. In this special case you may want to increase the acquire time. Increasing
          the acquire time will capture more rows of protocol data into the Link Training Results
          Table. Naturally, you will have to wait a little longer for the acquisition to complete.
          The Link Training application is only available on the 70KSX oscilloscopes.

    **Usage:**
        - Using the ``.query()`` method will send the ``LINKTRaining:ACQTime?`` query.
        - Using the ``.verify(value)`` method will send the ``LINKTRaining:ACQTime?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``LINKTRaining:ACQTime value`` command.

    **SCPI Syntax:**

    ::

        - LINKTRaining:ACQTime <NR1>
        - LINKTRaining:ACQTime?

    **Info:**
        - ``<NR1>`` is the maximum Acquire Time for Link Training from 1 to 10 seconds.
    """


class Linktraining(SCPICmdRead):
    """The ``LINKTRaining`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``LINKTRaining?`` query.
        - Using the ``.verify(value)`` method will send the ``LINKTRaining?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.acqtime``: The ``LINKTRaining:ACQTime`` command.
        - ``.armscope``: The ``LINKTRaining:ARMscope`` command.
        - ``.decode``: The ``LINKTRaining:DECOde`` command.
        - ``.equalizationch``: The ``LINKTRaining:EQUalizationCH<x>`` command.
        - ``.lane``: The ``LINKTRaining:LANE`` command.
        - ``.mark``: The ``LINKTRaining:MARK`` command.
        - ``.setup``: The ``LINKTRaining:SETUP`` command.
    """

    def __init__(
        self, device: Optional["PIDevice"] = None, cmd_syntax: str = "LINKTRaining"
    ) -> None:
        super().__init__(device, cmd_syntax)
        self._acqtime = LinktrainingAcqtime(device, f"{self._cmd_syntax}:ACQTime")
        self._armscope = LinktrainingArmscope(device, f"{self._cmd_syntax}:ARMscope")
        self._decode = LinktrainingDecode(device, f"{self._cmd_syntax}:DECOde")
        self._equalizationch: Dict[
            int, LinktrainingEqualizationchItem
        ] = DefaultDictPassKeyToFactory(
            lambda x: LinktrainingEqualizationchItem(
                device, f"{self._cmd_syntax}:EQUalizationCH{x}"
            )
        )
        self._lane = LinktrainingLane(device, f"{self._cmd_syntax}:LANE")
        self._mark = LinktrainingMark(device, f"{self._cmd_syntax}:MARK")
        self._setup = LinktrainingSetup(device, f"{self._cmd_syntax}:SETUP")

    @property
    def acqtime(self) -> LinktrainingAcqtime:
        """Return the ``LINKTRaining:ACQTime`` command.

        **Description:**
            - This 70KSX command sets the maximum Acquire Time for Link Training from 1 to 10
              seconds. The default value is 2 seconds. The link training process is defined by the
              IEEE to complete within 500 ms. So, the default 2 seconds is sufficient for most
              purposes. However, some devices can be put into a mode where the Link Training process
              is repeated continuously. In this special case you may want to increase the acquire
              time. Increasing the acquire time will capture more rows of protocol data into the
              Link Training Results Table. Naturally, you will have to wait a little longer for the
              acquisition to complete. The Link Training application is only available on the 70KSX
              oscilloscopes.

        **Usage:**
            - Using the ``.query()`` method will send the ``LINKTRaining:ACQTime?`` query.
            - Using the ``.verify(value)`` method will send the ``LINKTRaining:ACQTime?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``LINKTRaining:ACQTime value``
              command.

        **SCPI Syntax:**

        ::

            - LINKTRaining:ACQTime <NR1>
            - LINKTRaining:ACQTime?

        **Info:**
            - ``<NR1>`` is the maximum Acquire Time for Link Training from 1 to 10 seconds.
        """
        return self._acqtime

    @property
    def armscope(self) -> LinktrainingArmscope:
        """Return the ``LINKTRaining:ARMscope`` command.

        **Description:**
            - This 70KSX command sets or queries the ARMscope setting of the Link Training
              application on the 70KSX family of scopes. The default argument value is OFF. To use
              the Link Training application, the State argument must first be set to ON. With the
              DUT Lane 'powered down' (flat-line), you then Arm the scope for single-step capture
              using this command. When you subsequently initiate Link Training on the DUT
              ('power-up' the lane), the Control Channel data and digitized waveforms are captured
              by the scope. At a minimum, to capture the Link Training data into a Results Table,
              both the ``LINKTTRaining:STAte ON`` and ``LINKTRaining:ARMscope ON`` commands must be
              issued, in that order. When Link Training is initiated on the DUT lane, the Results
              Table data and digitized waveforms will be captured. You may select other Link
              Training settings using the related commands shown below to control those features.
              Initiating Link Training on the DUT is a DUT-dependent operation that is usually
              accomplished by a (serial or other bus) command to the DUT to power-up the Lane under
              test. Sometimes, it can be as simple as un-plugging and re-plugging the signal
              connector for the lane. Note that it is not necessary to set the ARMscope to OFF as
              this is done automatically by the system. To summarize, the minimum sequence of events
              to capture 100G Ethernet KR4 from Default Setup is ``:LINKTRaining:STATE ON``
              ``:LINKTRaining:ARMscope ON`` Initiate Link Training on the Lane under test on the
              DUT. The Results Table and Waveforms are captured on the scope. To repeat the
              experiment, all you have to do is power-down the lane on the DUT, re-issue the
              ARMscope command, and power-up the lane on the DUT again. Link Training is only
              available on the 70KSX oscilloscopes.

        **Usage:**
            - Using the ``.query()`` method will send the ``LINKTRaining:ARMscope?`` query.
            - Using the ``.verify(value)`` method will send the ``LINKTRaining:ARMscope?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``LINKTRaining:ARMscope value``
              command.

        **SCPI Syntax:**

        ::

            - LINKTRaining:ARMscope <OFF | ON>
            - LINKTRaining:ARMscope?

        **Info:**
            - ``OFF`` indicates ARMscope is OFF.
            - ``ON`` indicates ARMscope is ON.
        """
        return self._armscope

    @property
    def decode(self) -> LinktrainingDecode:
        """Return the ``LINKTRaining:DECOde`` command.

        **Description:**
            - This 70KSX command sets or queries the Decode setting of the Link Training
              application. The default argument value is ON, which decodes the Control Channel
              information on the waveform display. Other system decode parameters optionally allow
              the Control Channel Hex Decode to be expanded to also include bit-level decode. The
              only plausible reason to turn off decoding is to save time when you are capturing a
              very long record using TRIGgeron ``FIRst_frame`` or ``LASt_frame``. With TRIGgeron
              ``ALL_frames``, the overhead for decoding is very small. Link Training is only
              available on the 70KSX family of oscilloscopes.

        **Usage:**
            - Using the ``.query()`` method will send the ``LINKTRaining:DECOde?`` query.
            - Using the ``.verify(value)`` method will send the ``LINKTRaining:DECOde?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``LINKTRaining:DECOde value``
              command.

        **SCPI Syntax:**

        ::

            - LINKTRaining:DECOde <OFF | ON>
            - LINKTRaining:DECOde?

        **Info:**
            - ``OFF`` indicates decode is OFF.
            - ``ON`` indicates decode is ON.
        """
        return self._decode

    @property
    def equalizationch(self) -> Dict[int, LinktrainingEqualizationchItem]:
        """Return the ``LINKTRaining:EQUalizationCH<x>`` command.

        **Description:**
            - This 70KSX command sets the equalization value for Link Training of the specified
              channel. The default setting is zero. The units are dB. If you are probing a signal
              near to the transmitter, you likely do not need to increase the equalization value.
              However, if you are probing on a backplane, or other situation where the signal is
              seriously attenuated, you may want to increase the equalization value. Note that this
              equalization only applies to the trigger path in the oscilloscope. It does not apply
              to the data path. Use the SDLA option to provide equalization/de-embedding on the data
              path. Increasing the trigger path equalization value will improve the protocol data
              recovered into the Link Training Results Table when you are probing a seriously
              attenuated signal. Use the SDLA option to improve the equalization of the data path
              waveforms displayed on the oscilloscope. Since the protocol data (Ethernet Control
              Channel Coefficient Updates and Status Responses) is transmitted at ¼ the nominal bit
              rate (25 Gb/s for 100G Ethernet), the equalization value should be set considering ½
              that reduced bitrate. For example, for 100G Ethernet, the protocol data is transmitted
              at 6.25 Gb/s, so set the equalization value considering half of that rate, 3.125 Gb/s.
              The Link Training application is only available on the 70KSX scopes.

        **Usage:**
            - Using the ``.query()`` method will send the ``LINKTRaining:EQUalizationCH<x>?`` query.
            - Using the ``.verify(value)`` method will send the ``LINKTRaining:EQUalizationCH<x>?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``LINKTRaining:EQUalizationCH<x> value`` command.

        **SCPI Syntax:**

        ::

            - LINKTRaining:EQUalizationCH<x> <NR1>
            - LINKTRaining:EQUalizationCH<x>?
        """
        return self._equalizationch

    @property
    def lane(self) -> LinktrainingLane:
        """Return the ``LINKTRaining:LANE`` command.

        **Description:**
            - This command sets or queries the activation of the individual communications lanes
              under test. The defaults are Lane1 ON, and Lane2 OFF. It is taken as given that Lane1
              uses oscilloscope Ch1 and Ch3, Lane2 uses oscilloscope Ch2 and Ch4 (to maximize sample
              rate). ATI DPO70000SX oscilloscopes only support Lane1 (on Ch1 and Ch3). <X> is the
              lane number and can be 1 or 2. One or two lanes may be tested on a single 4-channel
              DPO70000SX oscilloscope. To test up to 4 lanes simultaneously, two oscilloscopes
              running in Standalone mode may be used. This is necessary because the Time-Sync
              Standalone mode does not support independent triggering by multiple oscilloscopes. The
              associated Link Training Arm PI command or UI button, may be used to arm each
              oscilloscope individually. When the Link Training signal is generated by the devices
              under test, the individual oscilloscopes will each trigger, capturing the waveforms on
              their respective displays and the protocol information in their respective Results
              Tables. Note that each Lane requires two oscilloscope channels, because the lanes are
              bi-directional (Full Duplex). Link Training is available on DPO70000SX oscilloscopes
              only.

        **Usage:**
            - Using the ``.query()`` method will send the ``LINKTRaining:LANE?`` query.
            - Using the ``.verify(value)`` method will send the ``LINKTRaining:LANE?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``LINKTRaining:LANE value`` command.

        **SCPI Syntax:**

        ::

            - LINKTRaining:LANE <x> {<NR1>|OFF|ON}
            - LINKTRaining:LANE?

        **Info:**
            - ``<NR1>`` = 0 disables the specified lane; any other value enables the specified lane.
            - ``OFF`` disables the specified lane.
            - ``ON`` enables the specified lane.
        """
        return self._lane

    @property
    def mark(self) -> LinktrainingMark:
        """Return the ``LINKTRaining:MARK`` command.

        **Description:**
            - This command sets or queries the Mark setting of the Link Training application. The
              default argument value is FRAme, which marks the Frame Marker. When the argument is
              set to CONtrol, the frame Control Channel information is marked (32 cells of
              differential-Manchester-encoded request/response data). When the argument is TRAining,
              the PRBS Training Data is marked. Marking the PRBS Training Data is useful for giving
              you the beginning/ending of the Training Data section of the frame when you want to
              use cursor-gated DPOJET measurements. Link Training is only available on the 70KSX
              family of oscilloscopes.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``LINKTRaining:MARK value`` command.

        **SCPI Syntax:**

        ::

            - LINKTRaining:MARK <FRAme | CONtrol | TRAining>

        **Info:**
            - ``FRAme`` sets Link Training MARK to FRAME.
            - ``CONtrol`` sets Link Training MARK to CONTROL.
            - ``TRAining`` sets Link Training MARK to TRAINING.
        """
        return self._mark

    @property
    def setup(self) -> LinktrainingSetup:
        """Return the ``LINKTRaining:SETUP`` command.

        **Description:**
            - This 70KSX command saves or restores custom Link Training setup files. The setup files
              are complete instrument setup files. The setup files are saved/restored depending on
              the current values for Link Training Standard, Trigger-On, and Lane selections. You
              only need to use this command when you desire to preserve custom Vertical, Horizontal,
              or Zoom settings for the particular Link Training mode you are using. When Link
              Training is enabled, you may dismiss the pop-up control window and make custom changes
              to the Vertical, Horizontal, or Zoom settings, for example, to accommodate various
              probes or record length requirements. If you want to save these settings for future
              use, just issue the ``LINKTRaining:SETUP SAVe`` command. You can restore the factory
              Link Training setup at any time by using the ``LINKTRaining:SETUP RESTore`` command.
              UI buttons for these operations are also supplied via the Customize button on the
              pop-up UI for Link Training. You do not need to enter any file or directory names to
              use these commands. When you save your custom setup with this command, the Link
              Training setup file pertaining to the current Standard, Trigger-On, and Lane
              selections is overwritten with your new setup. You may restore the factory Link
              Training setup file at any time. Again, these commands save/restore the complete
              instrument setup for the particular Link Training mode currently selected. The Link
              Training application is only available on the 70KSX scopes.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``LINKTRaining:SETUP value`` command.

        **SCPI Syntax:**

        ::

            - LINKTRaining:SETUP <SAVe |RESTore>

        **Info:**
            - ``SAVe`` saves your custom Link Training setup.
            - ``RESTore`` restores the custom Link Training setup.
        """
        return self._setup
